% ball4fr.m
%
% This program prepares frequency response graphs for computational results 
% from ball4.m. It is mainly bookeeping with tensors.
%
clear
load ball4 G1 G2 G3 ww

nw = length(ww);
[r1,c1,p1] = size(G1);
[r2,c2,p2] = size(G2);
[r3,c3,p3] = size(G3);

% for each page i=1:p1(the same as p2, p3), calculate the product of 
% Gi(w) and its complex conjugate. GGi is 4 by 4 by 100 array. Each page
% is 4 by 4 matrix, Gi(w)Gi(w)'

for i = 1:p1
   
   GG1(:,:,i) = G1(:,:,i)*G1(:,:,i)';
   GG2(:,:,i) = G2(:,:,i)*G2(:,:,i)';
   GG3(:,:,i) = G3(:,:,i)*G3(:,:,i)';
   
   % now calculate the determinant of G(w)G(w)'
   % GDi is 1 by 1 by 100 array. Each page is scalar: det(G(w)G(w)')
   
   GD1(:,:,i) = det(GG1(:,:,i));
   GD2(:,:,i) = det(GG2(:,:,i));
   GD3(:,:,i) = det(GG3(:,:,i));
   
   % now calculate KGi = det(Kappa - G(w)G(w)'), i represents case 2 and 3
   % KGi is 1 by 1 by 100 array
   
   sig = [-0.0,-0.1,-0.2];
   
   KG2(:,:,i) = det(-1/sig(2) - GG2(:,:,i));
   KG3(:,:,i) = det(-1/sig(3) - GG3(:,:,i));
   
   % calculate KLi = log(det(kappa-G(,)G(,)', i represents case 2 and 3
   
   KL2(:,:,i) = log(KG2(:,:,i));
   KL3(:,:,i) = log(KG3(:,:,i));
   
end

% GD is 3 by 100 matrix, each row represents one case for sig
% KG, KL are respectively 2 by 100 matrix, each row represents 
% sig = -0.1, sig=-0.2 respectively

GD = [reshape(GD1,1,p1);reshape(GD2,1,p2);reshape(GD3,1,p3)];

KG = [reshape(KG2,1,p2);reshape(KG3,1,p3)];

KL = [reshape(KL2,1,p2);reshape(KL3,1,p3)];


figure

for i = 1:3   
subplot(1,3,i)
plot(ww,GD(i,:))

if i == 1
legend('sig = -0.0')
elseif i == 2
   legend('sig = -0.1')
elseif i == 3
   legend('sig = -0.2')  
   set(gca,'YAxisLocation','right')
end

axis([0 pi -inf inf])
grid on
end

% add title to all the subplots

axes('visible','off')
xlabel('det(G(.)G(.)) as a function of w \in (0,\pi)','visible','on')


% plot kappa-** and log(kappa-**

for j=1:2
      
   figure
   for i = 1:2
   subplot(1,2,i)
   
   if j == 1
      plot(ww,KG(i,:)) 
   else
      plot(ww,KL(i,:))    
   end
     
   if i == 1     
      title('sig = -0.1')      
   elseif i == 2
      title('sig = -0.2')      
      set(gca,'YAxisLocation','right')
   end
   
   axis([0 pi -inf inf])   
   grid on
   
end

if j == 1
    axes('visible','off')
    xlabel('det(kappa - G(.)G(.)) as a function of w \in (0,\pi)','visible','on')     
 else
     axes('visible','off')
     xlabel('log det((kappa - G(.)G(.))) as a function of w \in (0,\pi)','visible','on')     
  end
end

  
