%  ball5.m
%  program to calculate risk sensitive solutions
%  of Larry  Ball's  NBER model
%  Version to do discounted model
%  disc is the discount factor
%
%

%  The model is
%
%  y_{t+1} = lambda y_t + (beta/theta) nu_t - (beta/theta + delta) e_t
%      + w_{1t+1}
%
%  pi_{t+1} = pi_t + alpha y_t + gamma e_{t-1}  - gamma e_t + w{2t+1}
%
%  nu_{t+1} =  w_{3t+1}   
%
%  The state is x_t =[y_t, pi_t, e_{t-1}, nu_t]
%
%  The control is e_t  
%  
%  Let r be the matrix in the x' R x piece of the return, 
%  and q =0 be the cost of control matrix.
%  We map this into a linear system, a, b, c, r, q.  We
%  set the discount factor 1.

%  parameter values
lambda=.8;
alpha=.4;
gamma=.2;
theta=2.0
beta=.6;
delta=.2;

disc=.9;    %  this is the discount factor, new to this version


mu=.5;    %   mu is weight on output variance
          %   (1-mu) is weight on inflation variance
a=[lambda 0 0 beta/theta;
   alpha 1 gamma 0;
   0 0 0 0;
   0 0 0 0];


b =[-(beta/theta + delta);
    -gamma;
      1;
      0];

a=a*sqrt(disc);
b=b*sqrt(disc);   %  the discount factor correction

c=[1 0 0;  
   0 1 0;
   0 0 0;
   0 0 sqrt(2)];   %  these are Ball's numbers from page 8  
                   %  note that they do not affect the sig=0 rule
		   %  but do the others


r=zeros(4); r(1,1)=mu;  r(2,2)=1-mu;


r(3,3)=.000001;    %  modifications of r to permit cholesky to exist. 
r(4,4)=.000001;

hh=chol(r);

q=.000001;     %  small adjustment costs for technical reasons



sig=-0.0;



   [k,v,vt]=doublex4(a',b',r,q,c,-sig);

%    remember that doublex.m solves a MIN problem, which requires
%    us to reverse the sign convention for sig

%   a11o=a11-b1*kg';      %a11o gives the 11 (all endogenous variables)
                         %part of ao.                                          


k1=k;
   ao=(a-b*k')


%   check from Ball's paper:

n=-k(2)
m=(-k(1)+alpha*k(2))/lambda

eps1=n*gamma+k(3)   %  should be zero

eps2=m*(beta/theta)+k(4)   %  should be zero


wball=m*beta*theta/(theta-m*beta+m*beta*theta);
aball=theta*(m*lambda+n*alpha)/(theta-m*beta+m*beta*theta);
bball=n*theta/(theta-m*beta+m*beta*theta);

div=wball+theta*(1-wball);  % divsor for impacts

aimpact=aball/div; 
bimpact=bball/div;


aimpact1=aimpact
bimpact1=bimpact

aball1=aball
bball1=bball

d=zeros(4,3);
sys1=ss(ao,c,hh,d,1);


I=eye(3);dm=zeros(3)
M=sig*((I - sig*c'*v*c)\c'*v*ao);   % M is the "G" for the distorted v_t from HST
ww=linspace(0,pi);   
G1=freqresp(sys1,ww);   %  chao: this is the frequency response (a complex
			%  number.


%   now compute system for second sig value


sig=-0.1;



   [k,v,vt]=doublex4(a',b',r,q,c,-sig);

%    remember that doublex.m solves a MIN problem, which requires
%    us to reverse the sign convention for sig

%   a11o=a11-b1*kg';      %a11o gives the 11 (all endogenous variables)
                         %part of ao.                                          

k2=k;

   ao=(a-b*k')


n=-k(2)
m=(-k(1)+alpha*k(2))/lambda

eps1=n*gamma+k(3)   %  should be zero

eps2=m*(beta/theta)+k(4)   %  should be zero


wball=m*beta*theta/(theta-m*beta+m*beta*theta);
aball=theta*(m*lambda+n*alpha)/(theta-m*beta+m*beta*theta);
bball=n*theta/(theta-m*beta+m*beta*theta);

div=wball+theta*(1-wball);  % divsor for impacts

aimpact=aball/div; 
bimpact=bball/div;


aimpact2=aimpact
bimpact2=bimpact

aball2=aball
bball2=bball




clear ss

sys2=ss(ao,c,hh,d,1);


I=eye(3);dm=zeros(3)
M=sig*((I - sig*c'*v*c)\c'*v*ao);   % M is the "G" for the distorted v_t from HST
ww=linspace(0,pi);   
G2=freqresp(sys2,ww);   %  chao: this is the frequency response (a complex

%   now compute system for third sig value

  sig=-.2;    %  original setting
%   sig=-0.20;     % experimental setting



   [k,v,vt]=doublex4(a',b',r,q,c,-sig);

%    remember that doublex.m solves a MIN problem, which requires
%    us to reverse the sign convention for sig

%   a11o=a11-b1*kg';      %a11o gives the 11 (all endogenous variables)
                         %part of ao.                                          



   ao=(a-b*k')


n=-k(2)
m=(-k(1)+alpha*k(2))/lambda

eps1=n*gamma+k(3)   %  should be zero

eps2=m*(beta/theta)+k(4)   %  should be zero


wball=m*beta*theta/(theta-m*beta+m*beta*theta);
aball=theta*(m*lambda+n*alpha)/(theta-m*beta+m*beta*theta);
bball=n*theta/(theta-m*beta+m*beta*theta);

div=wball+theta*(1-wball);  % divsor for impacts

aimpact=aball/div; 
bimpact=bball/div;


aimpact3=aimpact
bimpact3=bimpact

aball3=aball
bball3=bball


k3=k;



sys3=ss(ao,c,hh,d,1);


I=eye(3);dm=zeros(3)
M=sig*((I - sig*c'*v*c)\c'*v*ao);   % M is the "G" for the distorted v_t from HST
%ww=linspace(0,pi);   
G3=freqresp(sys3,ww);   %  chao: this is the frequency response (a complex

%  plot the maximum singular values
save ball5
return
