% ball5frd.m
%  computes "graphs" for ball model with discounting
%
% This program prepares frequency response graphs for computational results 
% from ball5.m. It is mainly bookeeping with tensors.
%


clear

load ball4frc

load ball5 G1 G2 G3 ww

nw = length(ww);
[r1,c1,p1] = size(G1);
[r2,c2,p2] = size(G2);
[r3,c3,p3] = size(G3);

% for each page i=1:p1(the same as p2, p3), calculate the product of 
% Gi(w) and its complex conjugate. GGi is 4 by 4 by 100 array. Each page
% is 4 by 4 matrix, Gi(w)Gi(w)'

for i = 1:p1
   
   GG1(:,:,i) = G1(:,:,i)*G1(:,:,i)';
   GG2(:,:,i) = G2(:,:,i)*G2(:,:,i)';
   GG3(:,:,i) = G3(:,:,i)*G3(:,:,i)';
   
   % now calculate the trace of G(w)G(w)'
   % GDi is 1 by 1 by 100 array. Each page is scalar: det(G(w)G(w)')
   
   GDd1(:,:,i) = trace(GG1(:,:,i));
   GDd2(:,:,i) = trace(GG2(:,:,i));
   GDd3(:,:,i) = trace(GG3(:,:,i));
   
   % now calculate KGi = det(Kappa - G(w)G(w)'), i represents case 2 and 3
   % KGi is 1 by 1 by 100 array
   
   sig = [-0.0,-0.1,-0.15];
   
   KG2(:,:,i) = det(-1/sig(2)*eye(4) - GG2(:,:,i));
   KG3(:,:,i) = det(-1/sig(3)*eye(4) - GG3(:,:,i));

%  in KGIJ(.,.,.) I refers to the rule and  J refers to the sigma
   KG10(:,:,i) = det( 1000*eye(4) - GG1(:,:,i));    %  big theta  
   KG12(:,:,i) = det(-1/sig(2)*eye(4) -GG1(:,:,i));  %  for   H_2
   KG23(:,:,i) = det(-1/sig(3)*eye(4) -GG2(:,:,i));   
   KG13(:,:,i) = det(-1/sig(3)*eye(4)- GG1(:,:,i));
   KG32(:,:,i) = det(-1/sig(2)*eye(4)- GG2(:,:,i));

   % calculate KLi = log(det(kappa*eye(4)-G(,)G(,)', i represents case 2 and 3
   
   KL2(:,:,i) = log(KG2(:,:,i));
   KL3(:,:,i) = log(KG3(:,:,i));
   KL10(:,:,i) = log(KG10(:,:,i)); 
   KL12(:,:,i) = log(KG12(:,:,i));  
   KL23(:,:,i) = log(KG23(:,:,i)); 
   KL13(:,:,i) = log(KG13(:,:,i));
   KL32(:,:,i) = log(KG32(:,:,i));
end


% GD is 3 by 100 matrix, each row represents one case for sig
% KG, KL are respectively 2 by 100 matrix, each row represents 
% sig = -0.1, sig=-0.2 respectively

GDd = [reshape(GDd1,1,p1);reshape(GDd2,1,p2);reshape(GDd3,1,p3)];

KG = [reshape(KG2,1,p2);reshape(KG3,1,p3)];

KL = [reshape(KL2,1,p2);reshape(KL3,1,p3)];

KKL=[reshape(KL12,1,p2);reshape(KL23,1,p3);reshape(KL13,1,p1);...
     reshape(KL32,1,p1)];


close all


figure(1)
subplot(2,1,1)


   hold on

   plot(ww,GD(1,:),'b-')
   plot(ww,GD(2,:),'g--')
   plot(ww,GD(3,:),'r-.')
%  gtext('\theta = \infty'),pause
%  gtext('\theta = 10'),pause
%  gtext('\theta = 5'),pause
   
hold off
subplot(2,1,2)


   hold on

   plot(ww,GDd(1,:),'b-')
   plot(ww,GDd(2,:),'g--')
   plot(ww,GDd(3,:),'r-.')
%  gtext('\theta = \infty'),pause
%  gtext('\theta = 10'),pause
%  gtext('\theta = 5'),pause
   
hold off

print -deps ballrobd.eps 

return

for j=1:2
      
   figure(j+2)
   for i = 1:2
   subplot(1,2,i)
   
   if j == 1
      plot(ww,KG(i,:)) 
   else
      plot(ww,KL(i,:))    
   end
     
   if i == 1     
      title('sig = -0.1')      
   elseif i == 2
      title('sig = -0.2')      
      set(gca,'YAxisLocation','right')
   end
%     axis([0 pi 0 10]) 
   axis([0 pi -inf inf])    %  Chao's original setting
   grid on
   
end

if j == 1
    axes('visible','off')
    xlabel('det(kappa - G(.)G(.)) as a function of w \in (0,\pi)','visible','on')     
 else
     axes('visible','off')
     xlabel('log det((kappa - G(.)G(.))) as a function of w \in (0,\pi)','visible','on')     
  end
end


figure
subplot(2,2,1),
plot(ww,[KL12(1,:)' KL13(1,:)'])
subplot(2,2,2)
plot(ww,[KL2(1,:)'  KL23(1,:)'])
subplot(2,2,3)
plot(ww,[KL3(1,:)' KL32(1,:)'])

[ww'  KL12(1,:)' KL13(1,:)']

%return
figure
plot(ww,[KL(2,:)'  KL23(1,:)'])


%
figure 
plot(ww,[KL12(1,:)' KL13(1,:)'])

figure
plot(ww,abs(KL10(1,:))','- b')
hold on
plot(ww,abs(KL(1,:))','-. r')
%plot(ww,KL13(1,:)','--')
hold off
title('rule 1 and two \theta ')
return
figure 
   plot(ww,[KL(2,:)' KL12(1,:)' KL23(1,:)']) 
