%  ballaut
%  altered version of ball4
%  for chicago fed paper
%
%  program to calculate risk sensitive solutions
%  of Larry  Ball's  NBER model
%  version to test the bode plot, April 17, 1998
%  also puts in some stuff for the freqresp for chao
%
%
%
%
%  The model is
%
%  y_{t+1} = lambda y_t + (beta/theta) nu_t - (beta/theta + delta) e_t
%      + w_{1t+1}
%
%  pi_{t+1} = pi_t + alpha y_t + gamma e_{t-1}  - gamma e_t + w{2t+1}
%
%  nu_{t+1} =  w_{3t+1}   
%
%  The state is x_t =[y_t, pi_t, e_{t-1}, nu_t]
%
%  The control is e_t  
%  
%  Let r be the matrix in the x' R x piece of the return, 
%  and q =0 be the cost of control matrix.
%  We map this into a linear system, a, b, c, r, q.  We
%  set the discount factor 1.

%  parameter values
lambda=.8;
alpha=.4;
gamma=.2;
theta=2.0
beta=.6;
delta=.2;



mu=.5;    %   mu is weight on output variance
          %   (1-mu) is weight on inflation variance
a=[lambda 0 0 beta/theta;
   alpha 1 gamma 0;
   0 0 0 0;
   0 0 0 0];


b =[-(beta/theta + delta);
    -gamma;
      1;
      0];

c=[1 0 0;  
   0 1 0;
   0 0 0;
   0 0 sqrt(2)];   %  these are Ball's numbers from page 8  
                   %  note that they do not affect the sig=0 rule
		   %  but do the others


r=zeros(4); r(1,1)=mu;  r(2,2)=1-mu;


r(3,3)=.000001;    %  modifications of r to permit cholesky to exist. 
r(4,4)=.000001;

hh=chol(r);

q=.000001;     %  small adjustment costs for technical reasons



sig=-0.0;



[f,p,pt]=doublex9(a,b,c,r,q,sig);
[vs]=value2(a-b*f,c,r+f'*q*f)

[v1,pf,con]=value1(a,b,c,r,q,f,.96,0)




[F1,K1,P1,P1t]=doublex9(a,b,c,r,q,sig1);  % the control problem

%  old code to check doublex9; check ok.
%[K1,P1,P1T]=doublex4(Astar',Bstar',R,Q,C,-sig1);
%F1=K1';
AA1=a-b*F1; % Matrix governing worst case shocks
[rc,cc]=size(c);

K1=-sig1*(inv(eye(cc)+sig1*c'*P1*c))*c'*P1*AA1; 
                 % Neng's formula, assume 
                 % P2>0
%Note: this will disagree with K1 from doublex9 when there is  discounting                
                 
% Form Cholesky decomposition of H'H 

   HH1=r+F1'*q*F1;   
   sHH1=sparse(HH1);
   sh1=cholinc(sHH1,1e-24);   % note h1 corresponds to sig1
   h1=full(sh1);

nog=100; % The number of grids
ran_sig2=linspace(0, -.15, nog);
%ran_sig2=linspace(0, .007, nog);

intout=zeros(nog,3);

for idx=1:nog;
   sig2=ran_sig2(idx);
   [F2,K2,P2,P2t]=doublex9(a,b,c,r,q,sig2);  % the control problem
   AA2=a-b*F2; 
   
% Now create K2, matrix governing worst case shocks

K2=-sig2*(inv(eye(cc)+sig2*c'*P2*c))*c'*P2*AA2; 
                % Neng's formula, assume 
                % P2>0
   % Now form the state space systems for sig1 and sig2
   % The systems matrices are in the paper. Compute the system
   % response appropriate under the sig1 rule when
   % sig2 generates worst case shocks.
   
   sys1=ss(sqrt(delta)*(AA1+c*K2),c,h1,zeros(rc,cc),1);

   % note: we are using the h1 matrix corresponding to the 
   % sig1 decision rule; while the K2 matrix corresponds to
   % the sig2 worst case shocks.
   % Now compute frequency response function
   
   ww=linspace(0,pi,500);
   G1=freqresp(sys1,ww);  
   nw = length(ww);

   for i=1:nw
      GD(i) = trace(G1(:,:,i)'*G1(:,:,i));      
   end;  
   
   crit1=-mean(GD);
   %%% [v,pf,con]=value1(a,b,c,r,q,f,beta,nnc)
   [crit2]=value2(a+c*K2-b*F1,c,r+F1'*q*F1);
   intout(idx,:)=[sig2 crit1 crit2];
end;


%plot(ww,GD)




