%  ballautd
% This program is to determine the appropriate range of
% \sigma for Larry Ball's NBER model
%
%  The model is
%
%  y_{t+1} = lambda y_t + (beta/theta) nu_t - (beta/theta + delta) e_t
%      + w_{1t+1}
%
%  pi_{t+1} = pi_t + alpha y_t + gamma e_{t-1}  - gamma e_t + w{2t+1}
%
%  nu_{t+1} =  w_{3t+1}   
%
%  The state is x_t =[y_t, pi_t, e_{t-1}, nu_t]
%
%  The control is e_t  
%  
%  Let r be the matrix in the x' R x piece of the return, 
%  and q be the cost of control matrix.
%  We map this into a linear system, a, b, c, r, q.  We
%  set the discount factor 1.

%  parameter values
lambda=.8;
alpha=.4;
gamma=.2;
theta=2.0;
beta=.6;
delta=.2;

mu=.5;    %   mu is weight on output variance
          %   (1-mu) is weight on inflation variance
a=[lambda 0 0 beta/theta;
   alpha 1 gamma 0;
   0 0 0 0;
   0 0 0 0];

b =[-(beta/theta + delta);
    -gamma;
      1;
      0];

c=[1 0 0;  
   0 1 0;
   0 0 0;
   0 0 sqrt(2)];   %  these are Ball's numbers from page 8  
                   %  note that they do not affect the sig=0 rule
		   %  but do the others

r=zeros(4); r(1,1)=mu;  r(2,2)=1-mu;
q=.000001;     %  small adjustment costs for technical reasons

ran_sig=linspace(0, -0.11, 23);
bres=zeros(length(ran_sig),4);

for idx=1:length(ran_sig)
sig=ran_sig(idx);
[f,k,p,pt]=doublex9(a,b,c,r,q,sig);
aa=a-b*f;

T=142;
nsim=20000;
initx=[4.6 .05 .05 .05]';
[pa pb detecp]=detection2(initx,aa,c,k,T,nsim);
bres(idx,:)=[sig detecp pa pb];
end;

save bautres bres;







