%  bayes2
%% This version DEDUCTS the beta*theta*w'w term to the objective in the
%% Bayesian problem. Things now match up.  See the note to Lars below where
%  the 22 block of QQ is defined.  
%  Montana plane ride
%  Constructs the robust permanent income model and compares it with the
%  Bayesian solution; decomposes the F and P matrices
%  The "ordinary" (nonrobust)  value function and decision rule are
%    fo, po
%  The robust value function and decision rule are computed in two ways:
%    Fd, Kd, Pd  (doublex9 -- the doubling algorithm with a trick for winding
%    and unwinding the discount factor;
%    fc, kc, pc  (cheap9 -- uses the olrp -- ordinary regulator -- for the stacked system using the trick
%    described in chapter 2 of the monograph
%  In addition, we use olrp and the "big" system to compute the Bayesian
%  solution with the (x,X) procedure -- so that the state space are twice
%  as big.  The decision rule and value function for the big problem are in
%   fb, pb
%  The model is \sum_{t=0}^\infty -.5(c_t-b)^2
%  s.t.
%  c_t + k_t = R k_{t-1} + y_t 
%  y_{t+1} = \mu_y (1-\rho) + rho y_t + cy (\epsilon_{t+1} + w_{t+1})
%  with risk-sensitivity parameter = sig
%
%  Two checks to perform 
%  1.  Fb=fb(1:3)+fb(4:6)
 % Fb-fc  % this should be zero
 % Fb-Fd  % this should be zero too.  Both of these check out
%  2. a. compare pb12=pb(1:3,1:3) with po.  They should be equal. They are.
%     b. compare pc=Pd with pb11+pb12+pb21+pb22.  They are NOT (`Lars's
%     conjecture seems to need modifying).  
 
 %

%  step 1: solve the robust linear regulator using doublex9

%  set parameters

 rho=.9;
 muy=5;
 beta=.95; 
 
 Rr=1/beta;
 cy=.05   ;  % innovation volatility
 b=30;  
 sig=-.05;  % risk-sensitivity parameter
 % form linear regulator
 
 A=[Rr -b 1;
    0 1 0 ;
    0 muy*(1-rho) rho];
 B = [-1 0 0]';
 
 C=[0 0 cy]';
 Q=-.0000001*eye(3);  R=-1;W=0;
 
 [fo,po]=olrp(beta,A,B,Q,R)
 
 %  Step 2: solve the robust regulator
 % do it two ways using cheap and doublex9
 

Ad=beta^(.5)*A;
Bd=beta^(.5)*B;
%Cd=beta^(.5)*C;
tic
[Fd,Kd,Pd,Ptd]=doublex9(Ad,Bd,C,Q,R,sig);
Kd=Kd/sqrt(beta)   % we have to unwind the sqrt beta transform to get
%%  K component


[fc,kc,pc,ptc]=cheap9(beta,A,B,C,Q,R,sig)



fc-Fd
kc-Kd


% step 3
%  form the Bayesian problem with the bigger system

A22=A-B*Fd+C*Kd;  % can use either kc or Kd because they are equal!

AA=[A  C*Kd ; zeros(3,3) A22];
%  AA=[A  C*Kd ; zeros(3,3) A22];
BB=[B;zeros(3,1)];


%% Lars: Here is the key change from bayes1: notice the sign. I have
%% DEDUCTED the beta*theta*X'*Kd*Kd*X term from the objective function
%% My dyslexia might be working here.  

theta=-1/sig;
QQ=-.0000001*eye(6);
%QQ=zeros(6);
QQ(4:6,4:6)=-beta*theta*Kd'*Kd;
RR=-1


 [fb,pb]=olrp(beta,AA,BB,QQ,RR)
 

 %  now do the checks
 
 Fb=fb(1:3)+fb(4:6)
 
 Fb-fc  % this should be zero
 Fb-Fd  % this should be zero
 pb
 po
 pb11=pb(1:3,1:3);
 pb12=pb(1:3,4:6);
 pb21=pb(4:6,1:3);
 pb22=pb(4:6,4:6);
 
 pball=pb11+pb12+pb21+pb22;
 
 
 pball
 
 pc
 
 pball-pc
 
 pball-ptc
 
 pball-Pd
 
 
disp('checks')

disp('fc-Fd')
fc-Fd
disp('kc-Kd')
kc-Kd
disp('po')
po
disp('pb')
pb
disp('pc')
pc
disp('Pd')
Pd
disp('pball')
pball

disp('fo -fb(1:3)')
fo-fb(1:3)

return
 
 %  now compute theta*\sum_{t=0}^\infty beta^{t+1} w_{t+1}' w_{t+1}
 
 
 Astar=sqrt(beta)*A22;
 
 VV=doublej(Astar',-beta*theta*Kd'*Kd)
 VW=doublej2(Astar',Kd',Astar',Kd');
 VW=-beta*theta*VW;
 %VVc=doublej(Astar',kc'*kc)
 %VVc=-beta*theta*VVc
 

 
 % Va=-beta*theta*VV
 Va=VV;
 pb22
 Va./pb22
 pb22./Va
 return
 
 pball-Pd
 
 (pball-Pd)./Va
 
 

 
pball-Pd-Va

pball-Pd+Va