function[ps,qs,Qs,ws,Ws]=compfringefn(thm,thf);
% function[ps,qs,Qs,ws,Ws]=compfringefn(thm,thf);
% thm = theta for monopolist
% thf = theta for competitive firm
%  function version of compfring program  
% compfringe.m  August 28, 2005
%  take account of the fact that doublex9 asks for a MINIMUM problem and therefore treats the second problem appropriately.
%  Please notice the SIGNS on the Q and R. 
% written to explore the "Richard Dennis problem"
% August 25, 2005, London WC.  We have to remember that
% we are solving a maximum problem and that doublex9 is set up for a
% minimum problem.  A negative sigma in doublex9 corresponds to more
% robustness for the minimum problem. 
%
% There is a representative competitive firm with cost function
%  d q_t + .5 h q_t^2 + .5 c(q_{t+1} - q_t)^2 
% A large firm has cost function
%  eQ_t + .5 g Q_t^2 + .5 c(Q_{t+1} - Q_t)^2
% Demand is 
% p(t)= ao-a1*(Q(t)+q(t))+v(t)
% v(t+1)= rho*v(t)+Ce*eps(t+1); eps is iid(0,1)
%
% Equilibrium concept: Stackelberg, with mu_{xt} being the implementation
% multiplier. 
% the state vector in the final representation is
% y_t' = [1 v_t V Q_t q_t \mu_{it} mu_{wt}]' (7 \times 1)

     a0= 100;    % intercept of demand function
     a1=1;      % slope of demand function
     rho=0.8;   % autocorrelation factor  % originally set at .8
     Ce=.2;     % variance of shocks
     c=10;      % adjustment cost factor  % originally set at 1
     b=0.95;    % discount factor
     
     d=20;      % unit cost for competitive firm
     e=20;      % unit cost for large firm
     g=1;     %  quadratic term in cost function for large firm; originally set at .2
     h=1;      % quadratic term in cost function for competitive firm% originally set at .2
     
%      thm=2000000;   % theta for monopolist
%      thf=3000000;   % theta for fringe


% 

%%function[y1,xinitial,xs,yinitial,ys,nn]=oligop2005a(thm,thf)  % uncomment


% Define the following matrices
     AA1=[1 0 0 0 0 0 0;0 1 0 0 0 0 0 ;0 0 1 0 0 0 0;0 0 0 1 0 0 0;...
          0 0 0 0 1 0 0;
             (a0-d) 1 0 -a1 -a1-h c 0;...
              0 0 0 0 -(1/(2*thf))*Ce 0 b*rho];  % Sargent corrected sign after rechecking Euler equation
%             0 0 0 0 (1/(2*thf))*Ce 0 b*rho];  % Sargent   put in the old sign that he had -- it seems that the `correction' was wrong!  London, Aug. 24
                %   corrected sign after rechecking Euler equation
     AA2=[1 0 0  0 0  0 0;0 rho 0 0 0 0 Ce;0 0 rho 0 0 0 0;...
             0 0 0 1 0 0 0;0 0 0 0 1 1 0;0 0 0 0 0 (c/b) 0 ; 
         0 0 0 0 0 0 1];
        % tjs changed last entry from b/c to c/b; see the notes
     BB=[0;0;0;1;0;0;0];
     CC=[0; 0;Ce;0;0; 0;0];
     
     A=inv(AA1)*AA2;
     Bb= inv(AA1)*BB;
     C = inv(AA1)*CC;
     B=inv(AA1)*[BB CC];
     

     Q= -[0 0 0 ((a0-e)/2) 0 0 0;0 0 0 0 0 0 0 ;...
         0 0 0 .5 0 0 0;((a0-e)/2) 0  .5 -a1-g/2 -(a1/2) 0 0; 0 0 0 -(a1/2) 0 0 0;...
         0 0 0  0 0 0 0;...
         0 0 0 0 0 0 0];
     Rs=c/2;
     
% Define the number of 'jump variables'.

     nx=2;
     n= max(size(Q));
     nz=n-nx;


 
  As=A*sqrt(b)
  Bs=Bb*sqrt(b)

  %  Q and R are not affected 
  
  Cs=C;
  
  [Fs,Ks,Ps,Pst]=doublex9(As,Bs,Cs,Q,Rs,-1/thm);  % flip sign of sig because we have a max problem
  Ks=b^(-.5)*Ks;
  
    
  Fnew(2,:)=-Ks;
  Fnew(1,:)=Fs;
  
  F=Fnew;
  P=Ps;  

  

% Manipulate matrices

P21=P(nz+1:n,1:nz);
P22=P(nz+1:n,nz+1:n);
F1 =F(1,:) ;
F2 =F(2,:);
A1 =A(1:nz,:);
A2 =A(nz+1:n,:);
B1 =Bb(1:nz,:);
B2 =Bb(nz+1:n,:);
C1 =C(1:nz,:);
C2 =C(nz+1:n,:);

%%%======================================

%disp('Law of motion cal M for z, mu_x = '),
 
MW=[eye(nz) zeros(nz,n-nz); P21 P22]*[A1-B1*F1-C1*F2 ;A2-B2*F1-C2*F2]*[eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)];
CM0=[0 0 0 0 0 0 0]';
CM1= [eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)]*CM0;


% Now set up the matrices to formulate the recursive version of the
% representative firm's problem:

eQ=[0 0 0 1 0 0 0];  % selects Q
eq=[0 0 0 0 1 0 0];  % selects \bar q
e1=[1 0 0 0 0 0 0];  % selects 1
Rxq = ((a0-d)*e1-a1*(eq+eQ))/2;

AM=[MW zeros(7,2);
    zeros(1,7) rho 0;
    zeros(1,7) 0 1];


BM= [zeros(8,1);1];
CM=[CM1;Ce;0];


QM=[zeros(7,7) zeros(7,1) Rxq';
    zeros(1,7) 0 .5;
    Rxq .5 -.5*h];
    
RM=-.5*c;




% Solve the optimal linear regulator from the above matrices
RMM=[-c/2 0; 0 -b*thf];
% BM= [zeros(8,1);1];
% CM=[CM1;Ce;0];
BMM=[BM CM];
esmall=.0000001;
QMM=QM+eye(9)*esmall;
tic;     
[Fo Po]= olrp(b,AM,BMM,QMM,RMM);
toc;




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% Now solve the representative firm's decision problem:
% Use the trick to adjust for discounting

Ms=AM*sqrt(b)
  Bs=BM*sqrt(b)
  %  Q and R are not affected 
  %Rs=c/2;
  %Cs=C;

  [FM,KM,PM,PMt]=doublex9(Ms,Bs,CM,-QM,-RM,-1/thf);
  
  KM=b^(-.5)*KM;
  
 % adjust the sign on F because of the +, - convention for the K, M
 
  FM=-FM;
  
% now compare the decision rules for u_t, w_{t+1} from the two approaches.
% from solving the Stackelberg problem, the decision rules are embedded in
% 
MM= [-inv(P22)*P21 , inv(P22)];  

MMcon=zeros(2,7);

MMcon(1,:)=FM(1:7);
MMcon(1,2)=FM(2)+FM(8);   % add the coefficients on v
MMcon(1,5)=FM(5)+FM(9);   % add the coefficients on q

MMcon(2,:)=KM(1:7);
MMcon(2,2)=KM(2)+KM(8)  ;   % add the coefficients on v
MMcon(2,5)=KM(5)+ KM(9) ;   % add the coefficients on q 

disp('thf, thm ='), thf, thm

MM
MMcon   %  MMcon is the "comparison" object formed from the recursive c.e.
disp('MM./MMcon should be vector of 1 s ='),MM./MMcon

disp('with u the first row and w the second row')
disp('the coefficients on the state   [1 v_t V Q_t q_t \mu_{it} mu_{wt}] = MM are'),MM

%disp('Law of motion cal M for z, mu_x = '),
 
MW=[eye(nz) zeros(nz,n-nz); P21 P22]*[A1-B1*F1-C1*F2 ;A2-B2*F1-C2*F2]*[eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)]

% CM0=[0 0 0 0 0 0 0]';  
% 
% CM1= [eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)]*CM0;



%  now deduce the law of motion under the approximating model



Mapprox0(1,:)=MW(1,:);  % constant

Mapprox0(2,:)=MW(2,:)+MW(3,:)   %  v and V
Mapprox0(3:6,:)=MW(4:7,:);   % remaining variables Q q mu_w mu_i 


Mapprox(:,1)=Mapprox0(:,1);   % coeff on constan
Mapprox(:,2)=Mapprox0(:,2)+Mapprox0(:,3);   % coefficient on v 
Mapprox(:,3:6)=Mapprox0(:,4:7);    % remaining variables  Q q mu_w mu_i

% set law of motion for v equal to law of motion under the approximating
% model

Mapprox(2,:)=[0 rho 0 0 0 0];

Ao=Mapprox    %  the law of motion for [1 v Q, q mu_w mu_i] under the approximating model

Co=[0 Ce 0 0 0 0]'

disp('Ao, Co is first row of state space system for approximating model -- construct G and D below')

%  Now get the formulas for the worst case u, w under the approximating
%  model by setting v=V; MWo is the pertinent matrix

MWo(:,1)=MW(:,1);   % coefficient on constant
MWo(:,2)=MW(:,2)+MW(:,3);  % coefficient on v
MWo(:,3:6)=MW(:,4:7);     % coefficients on Q q mu_w mu_i

disp('decision rule for u, w MW='),MW
disp('decision rule for u, w MWo with v=V is '),MWo


% now get the decision rule for U, W under the approximating model by
% setting v=V ; BigMWo is the pertinent matrix

BigMW=[-Fs; Ks]*[eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)] 
BigMWo(:,1)=BigMW(:,1);   % coeff on constant
BigMWo(:,2)=BigMW(:,2)+BigMW(:,3);  % coeff on v
BigMWo(:,3:6)=BigMW(:,4:7);    % coefficients on Q, q, mu_w, mu_i

disp('BigMW ='), BigMW
disp('BigMWo = '), BigMWo



%%%=============================================

%%%%%% new part of program 
 


% Part I.  Use sys to form state space system.  Then get impulse
% response and simulation of system.



%
%  (this transformation follows from the transformation 
%  from [z_t, x_t] to [z_t, \mu_{xt}]  .  See chapter 10.  
%
%  get state space system for p, q, Q, mu_u, mu_w, v,   W, w
%                             1  2  3   4     5    6    7 8  (numbers
%                             are position in Y vector)
%
G=[a0  1 -a1 -a1 0 0;    % picks off p
   0  0 0 1 0 0;         % selects q
   0  0 1 0 0 0;          % selects Q
   0  0 0 0 1 0;     % selects \mu_u 
   0  0 0 0 0 1;     % selects \mu_w;
   0 1  0 0 0 0;     % selects v;
   BigMWo(2,:);   % picks off W_{t+1}, worst case shock of monopolist
   MWo(2,:)]   ;  %  picks off w_{t+1}, worst case shock of competitive firm              
 % The observation vector is p, q, Q, mu_u, mu_w, v, u, W                

% now form the state space system M, Ct, G, DD=zeros(9,1)

DD=zeros(8,1);
sys1=ss(Ao,Co,G,DD,1) ;   % system for approximating model


% now construct the steady state objects


disp('thetam, thetaf= ' ),thm,thf
%disp('steady state of price, q, Q')
%  construct steady state

xs=null(eye(6)-Ao);
xs=xs./xs(1);
%    
disp(' xs is steady state value of  [1 v  Q q \mu_u \mu_w]'),
xs

disp('steady state price = ')

ps=[a0 1 -a1 -a1 0  0]*xs
disp('steady state q is')

qs=[0  0 0 1 0 0]*xs

disp('steady state Q is')
Qs=[0   0 1 0  0 0]*xs

disp('steady state xs'),xs

disp('steady state values of  p, q, Q, mu_u, mu_w, v,   W, w'),
yss=G*xs

ps=yss(1);
qs=yss(2);
Qs=yss(3);
ws=yss(8);
Ws=yss(7);

%xinitial=xs; xinitial(6)=0; xinitial(7)=0;   % start from zero shadow prices; 
%                               %  all other state variables from steady state
%                               
% disp('initial condition for state'),xinitial        

% 
% disp('steady state values of p, q, Q, mu_u, mu_w, v, V, u, W, w'),G*xs  
% disp('initial values of  p, q, Q, mu_u, mu_w, v, V, u, W, w'),G*xinitial
% ys=G*xs;
% yinitial=G*xinitial;  
% 


return


 
%  now compute impulse response functions for p, q, Q, u with respect
% to the scalar demand shock epsilon.  

nn=12;   %  mm is length of impulse response
y1=impulse(sys1,nn);  y1=y1(2:nn,:);
y2=impulse(sys2,nn);  y2=y2(2:nn,:);






disp('thetam, thetaf= ' ),thm,thf
%disp('steady state of price, q, Q')
%  construct steady state

xs=null(eye(7)-M);
xs=xs./xs(1);
%    
%   xs is steady state value of  [1 v V Q q \mu_u \mu_w];  


ps=[a0 0 1 -a1 -a1 0  0]*xs
qs=[0 0 0 0 1 0 0]*xs
Qs=[0 0  0 1 0  0 0]*xs

disp('steady state xs'),xs

xinitial=xs; xinitial(6)=0; xinitial(7)=0;   % start from zero shadow prices; 
                              %  all other state variables from steady state
                              
disp('initial condition for state'),xinitial        


disp('steady state values of p, q, Q, mu_u, mu_w, v, V, u, W, w'),G*xs  
disp('initial values of  p, q, Q, mu_u, mu_w, v, V, u, W, w'),G*xinitial
ys=G*xs;
yinitial=G*xinitial;  

return
figure(1)

tt=[0:nn-2];  % for cosmetics of figure, to label x-axis properly

figure(1)   %  impulse response
subplot(4,1,1)
stairs(tt,y1(:,1)),ylabel('p')
subplot(4,1,2)
stairs(tt,y1(:,2)),ylabel('q')
subplot(4,1,3)
stairs(tt,y1(:,3)),ylabel('Q')
subplot(4,1,4)
stairs(tt,y1(:,2)+y1(:,3)),ylabel('q +Q')

print -depsc monopoly1

figure(2); % impulse response 
subplot(2,1,1)
stairs(tt,y1(:,5)),ylabel('\mu_w')
subplot(2,1,2)
stairs(tt,y1(:,4)),ylabel('\mu_u')
print -depsc monopoly2
figure(3);  % impulse respons3
subplot(3,1,1)
stairs(tt,y1(:,6)),ylabel('v')
subplot(3,1,2)
stairs(tt,y1(:,7)),ylabel('V')
subplot(3,1,3)
stairs(tt,y1(:,7)-y1(:,6)),ylabel('V-v')
print -depsc monopoly3

figure(4)  % impulse response

subplot(2,1,1)
stairs(tt,y1(:,9)),ylabel('W')
subplot(2,1,2)
stairs(tt,y1(:,10)),ylabel('w')

print -depsc monopoly4



%  Now compute simulation, starting from appropriate initial state.



TT=50+1;
u=randn(TT,1);

tp=[0:TT-1];

return
%  now simulate and prepare plots

Y1=lsim(sys1,u,[],xinitial);

figure(5)
subplot(3,1,1)
plot(tp,Y1(:,2)+Y1(:,3),'-g'),ylabel('q+Q')
subplot(3,1,2),
plot(tp,Y1(:,2),'-b'),ylabel('q')
subplot(3,1,3)
plot(tp,Y1(:,3),'--r'),ylabel('Q')
print -depsc monopoly5

figure(6)
subplot(3,1,1)
plot(tp,Y1(:,4),'-g'),ylabel('\mu_u')
subplot(3,1,2)
plot(tp,Y1(:,5),'-b'),ylabel('\mu_w')
subplot(3,1,3)
plot(tp,Y1(:,1),'--r'),ylabel('p')
print -depsc monopoly6


figure(7)
subplot(3,1,1)
plot(tp,Y1(:,9),'-g'),ylabel('W')
subplot(3,1,2)
plot(tp,Y1(:,10),'-b'),ylabel('w')
subplot(3,1,3)
plot(tp,Y1(:,9)-Y1(:,10),'--r'),ylabel('W-w')
print -depsc monopoly7

figure(8)
subplot(3,1,1)
plot(tp,Y1(:,7),'-g'),ylabel('V')
subplot(3,1,2)
plot(tp,Y1(:,6),'-b'),ylabel('v')
subplot(3,1,3)
plot(tp,Y1(:,7)-Y1(:,6),'--r'),ylabel('V-v')
print -depsc monopoly8

return
