%  decentr.m
% checks out decentralization for pure endowment economy
%  for the chapter on decentralization of the robust planning economy
%

% As an example, set $\beta =.99, \theta = .001^{-1}$, d_t - .7
% d_{t-1} = (1-.7)5 + .05 w_t$, b_t =30$.  Let the state be $x_t =
% \left[\matrix{1 \cr d_t\cr} \right]$.  Then we compute that
% $\tilde K_1 = K = \left[\matrix{ -0.0044  &  0.0001 \cr}\right]$
% and $\tilde K_2 =0$.
% 
% in the program below, the state for the household problem is $X, x$ 
% in that order


%  first set up the parameters

bet=.99;
rho=.7;
mud=5;
mub=30;
A=[1 0 ; (1-rho)*mud rho];
Ub=[30 0];
Ud=[0 1];
c=.05;

C=[0 c]';

R=(Ud-Ub)'*(Ud-Ub);    
M=2*(Ub-Ud);

% Note that M'*M/4=R    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


sig=-.001;
thet=-sig^(-1);

%%% set up planning problem


Q1=-R;
R1=bet*thet
B1=C;
A1=A;
[f1,p1] = olrp(bet,A1,B1,Q1,R1);

K=-f1 %  this is the feedback for the planning problem

Aclosed=A+C*K;

A2=[Aclosed zeros(2,2); zeros(2,2) A];

Q2=[.25*M'*M .5*M'*(Ud-Ub);
    .5*(Ud-Ub)'*M zeros(2,2)];

B2=[zeros(2,1); C];

R2=R1;

[f2,p2]=olrp(bet,A2,B2,Q2,R2);

K2=-f2

disp('feedback K from planning problem='), K

disp('feedback K from household problem='),K2



disp('this works now!')
 return


Q3=[-.75*M'*M -.5*M'*(Ud-Ub);
    -.5*(Ud-Ub)'*M zeros(2,2)];

[f3,p3]=olrp(bet,A2,B2,Q3,R2);

K3=-f3