% Calculates the detection probability given input
% matrices
% A is the closed An0=A-B*F
% C is the vol matrix
% D is the worst case shock worstv_dbl coefficient \Delta
% T is the dimension of the time series to simulate
% init is the initial condition of the time series to
% simulate, x0 is taken in here
% nsim is the number of simulations to perform

function [pa, pb, detecp]=detection2(initx,A,C,D,T,nsim)

[rc,cc]=size(C);
nx=size(A,1);

hatA=A+C*D;

x=zeros(nx,T);
x2=zeros(nx,T);

x(:,1)=initx;
x2(:,1)=initx;

r_A=zeros(1,nsim);
r_B=zeros(1,nsim);

for ii=1:nsim
  wn=randn(cc,T);
  epsn=randn(cc,T);

  % assuming the approximating model is DGP;
  for tt=1:T-1
    x(:,tt+1)=A*x(:,tt)+C*wn(:,tt);
  end;

  v=D*x;

  % compute v \cdot v for given all t and 1st pt of r_A;
  tmp1=v.^2;
  tmp2=sum(tmp1,1);
  tmp3=mean(tmp2);
  
  pt11=tmp3/2;
  
  % compute v \cdot w for given all t and 2nd pt of r_A;
  
  cross_vw=zeros(1,T);
  
  for tt=1:T
    cross_vw(tt)=v(:,tt)'*wn(:,tt);
  end
  
  pt12=mean(cross_vw);
  
  % calc. r_A;
 
  r_A(ii)=pt11-pt12;


  % PART TWO

  % assuming the worst case model is DGP;
  for tt=1:T-1
    x2(:,tt+1)=hatA*x2(:,tt)+C*epsn(:,tt);
  end
  
  v2=D*x2;

  % compute v2 \cdot v for given all t and 1st pt of r_B;
  tmp4=v2.^2;
  tmp5=sum(tmp4,1);
  tmp6=mean(tmp5);
  
  pt21=tmp6/2;

  % compute v2 \cdot \eps for given all t and 2nd pt of r_B;
  
  cross_veps=zeros(1,T);
  
  for tt=1:T
    cross_veps(tt)=v2(:,tt)'*epsn(:,tt);
  end;

  pt22=mean(cross_veps);

  % calc. r_B;

  r_B(ii)=pt21+pt22;

end

na=0;
nb=0;
for jj=1:nsim
  if r_A(jj) <= 0
    na=na+1;
  end
end

pa=na/nsim;

for jj=1:nsim
  if r_B(jj) < 0
    nb=nb+1;
  end
end

pb=nb/nsim;

detecp=(1/2)*(pa+pb);



