%  lplqfinal.m
%  robust version of Lucas-Prescott investment under uncertainty model
%  Compute equilibrium in two ways and check consistency

%% Lucas Prescott model with robustness.  

     a0= 100;    % intercept of demand function
     a1=1;      % slope of demand function
     rho=0.9;   % autocorrelation factor  % originally set at .8
     Ce=.2;     % variance of shocks
     c=10;      % adjustment cost factor  % originally set at 1
     b=0.95;    % discount factor
     d=20;      % unit cost for competitive firm
     e=20;      % unit cost for large firm
     g=1;     %  quadratic term in cost function for large firm; originally set at .2
     h=1;      % quadratic term in cost function for competitive firm% originally set at .2
     
     thf=30;
 
 %  August 3: 10:15 p.m.  Step 3 is the setup that works -- you have to use
 %  the larger state space with distorted X in state space.  
  
 % step 1: solve the robust planning problem
 %
 % state is [1 q_t v_t]
 % 
 %  
 A=[1 0 0;
     0 1 0;
     0 0 rho];
 B=[0 1 0]';
 C=[0 0 Ce]';
 
 Q=[ 0 .5*(a0-d) 0;
     .5*(a0-d) -.5*(a1+h) .5;
     0    .5 0 ];
%  Q=Q-.000001*eye(3);   % trick to pump up rank
 R=-.5*c;
 
 As=A*sqrt(b)
  Bs=B*sqrt(b)
 [Fs,Ks,Ps,Pst]=doublex9(As,Bs,C,Q,R,-1/thf);
 Ks=b^(-.5)*Ks;
 Fs
 Ks
 
 
 % Solve the optimal linear regulator from the above matrices
RMM=[-c/2 0; 0 -b*thf];
% BM= [zeros(8,1);1];
% CM=[CM1;Ce;0];
BMM=[B C];
esmall=.00001;
QMM=Q+eye(3)*esmall;
tic;     
[Fo Po]= olrp(b,A,BMM,QMM,RMM);
toc;


%  
% Now compute the worst case law of motion of \bar q

Aoo=A-B*Fs+C*Ks;   %  this computes the worst case law of motion for [1 q v];



%%% end of step 1


 %  step 2: solve the representative firm's problem as a recursive problem:

%%  the state is [1 qbar vbar q v] (see the
%% monograph).   

% Form law of motion for qbar, vbar in the big X.
% first qbar
dqbar=zeros(1,5);
dqbar(1)=-Fs(1);   % constant
dqbar(2)=1-Fs(2);  % coeff on qbar
% dqbar(5) = -Fs(3); % coeff on vbar
dqbar(3) = -Fs(3); % coeff on vbar

% Form law of motion for vbar.  
% dvbar=zeros(1,5);
% dvbar(1)=Ks(1);    % constant
% dvbar(2)=Ks(2);    % coeff on qbar
% dvbar(3)=Ks(3);    % coeff on vbar

% Form law of motion for vbar.  -- This is the correct one.  
dvbar=zeros(1,5);
dvbar(1)=Aoo(3,1);
dvbar(2)=Aoo(3,2);
dvbar(3)=Aoo(3,3);

Aa=[1 0 0 0 0;
    dqbar;
    dvbar;
    0 0 0 1 0;
    0 0 0 0 rho];

Ba=[0 0 0 1 0]';

Ca=[0 0  0 0 Ce]';   % note: the worst case shock disturbs v but not \bar v -- notice how this
                     % conforms to the decentralization in the monograph

Qa=[0 0  0 .5*(a0-d) 0 ;
     0 0 0 -a1/2 0;
    0 0 0 0 0 ;
    .5*(a0-d) -a1/2 0 -.5*h .5;
    0 0 0 .5 0];
 Ra=-.5*c; 
 
  Afsa=Aa*sqrt(b);
  Bfsa=Ba*sqrt(b);
  
  


 [Ffa,Kfa,Pfa,Pfat]=doublex9(Afsa,Bfsa,Ca,Qa,Ra,-1/thf);
 Kfa=b^(-.5)*Kfa;
  
% now perform the checks comparing the planning problem and the law of motion implied by the 
% representative firm's choice by setting q=\qbar and v = \vbar and
% consolidating  
 
 Fcheck2=zeros(1,3);
 
 Fcheck2(1)=Ffa(1);
 Fcheck2(2)=Ffa(2)+Ffa(4);
 Fcheck2(3)=Ffa(3)+Ffa(5);
 
 
 Kcheck2=zeros(1,3);
 
 Kcheck2(1)=Kfa(1);
 Kcheck2(2)=Kfa(2)+Kfa(4);
 Kcheck2(3)=Kfa(3)+Kfa(5);
 

 Fcheck2
 Fs
 disp('Fcheck2 - Fs (should be zero) = '),Fcheck2-Fs
 
 Kcheck2
 Ks
 ('Kcheck2 - Ks (should be zero) = '),Kcheck2-Ks
return

