 % managrs2.m 
 % Corrected March 24, 2000 to have beta0/4, as Kelly suggested
 % robust filtering and control of Rudebusch-Svennson model
%----------------------------------------------------------------------
clear all
% enter parameter values

sig=0;  % risk-sensitivity parameter, just for purpose of K-filter

nu=.5; 	           % interest rate smoothing; RS use 0.5
lambda=1;                    % output gap parameter; RS use 1
delta=.96;                     % discount rate
alphas=[.14 .7 -.1 .28 .12]; % Phillips curve parameters
betas=[.1 1.16 -.25];        % IS curve parameters

beta0star=.01* betas(1)/4;  % rescale the parameters to account for units ; 
                            % corrected March 24 (divided by 4)
alpha0star= 100*alphas(1);

%  check alpha parameter values from paper

%  set st dev's of three noises

siggam=.0002;               % st.dev. of trend income growth innovation
sigy=.819;    % st dev of epsilon_y
sigpi=1.009;  % st dev of epsilon_pi




% set up system matrices, in notation of HS manuscript 

Aff=[ zeros(1,8);
   1 zeros(1,7);
   0 1 zeros(1,6);
   zeros(1,3) 1 zeros(1,4);
   zeros(1,3) 1 zeros(1,4);
   zeros(1,4) 1 zeros(1,3);
   zeros(1,8);
   zeros(1,6) 1 0];


Afy=[1 0;
   zeros(5,2);
   0 1; 
   0 0 ];

Ayf=[alphas(3:5) zeros(1,5);
   beta0star*ones(1,3)  -2*beta0star -beta0star ...
      -beta0star betas(3) 0];

Ayy=[alphas(2) alpha0star;
    beta0star betas(2)];

Afz=[zeros(8,2)];

Ayz=[-alpha0star 0;
    2-betas(2) -(1+betas(3))];

Azf=zeros(2,8);

Azy= zeros(2,2);

Azz=[2 -1;
   1 0 ]; 



% Now create A, Af, Ay, Az, from the manuscript

A = [Aff Afy Afz;
   Ayf Ayy Ayz;
   Azf Azy Azz];
   
Af= [Aff Afy Afz ]; 
Ay= [Ayf Ayy Ayz ]; 
Az=[Azf Azy Azz];

Bf=[zeros(1,3) 1 zeros(1,4)]';
   
By=[ 0 ;-beta0star];
   
Bz=zeros(2,1);


Cf=zeros(8,3);
Cy=[sigpi 0 0;
   0 .01*sigy siggam];
Cz=[0 0 siggam ;
   0 0 0 ];

C=[Cf; Cy; Cz];

B=[Bf; By; Bz];



%  compute objects from the Kalman filter, following the manuscript

[cchky, cchkz, ctldz]=filfnc(Azz,Ayz,Cz,Cy);


% create the Cbar for the filtered representation  

ny=2;
nz=2;
nl=8;
nu=1;


nvchk=ny;
nvtld=nz;
nx=nl+ny+nz;
nu2=nu+nvchk+nvtld;
n=nx+nu2;

% turn in the Cchk;

Cbar=[zeros(nl,nvchk+nvtld); cchky zeros(ny,nvtld); cchkz ctldz];


disp('the kalman gain and chol factorization');

cchky

cchkz

ctldz

Cbar

%  Now create the return function
%  x'H'H x + Q u

H1=[0 0 0 0 0 0 -2*(lambda^(.5)) lambda^(.5) 0 lambda^(.5) 0 0]';
     % picks off difference in growth rate of output
H2=[1 1 1 0 0 0 0 0 1  0  0 0]';

%H1= lambda^(.5)*[ zeros(1,9) 1 -1 0]';   % alternative H1 that picks off g_t = q_t - q_t^*

Q = nu;

R=(H1*H1' + H2*H2');


% first transform A, B


Astar=sqrt(delta)*A;
Bstar=sqrt(delta)*B;


% We now have the system in the right form to use 


% doublex9 (control problem) to solve the Riccati equations


%--------------------------------------------------

% set alternative values of the risk-sensitivity parameter

sig2=-.01;

sig2=-.01/4;   %  reset to adjust for /4 division correction


sig1=0;

%  first, form the system assuming no measurement error.  This
%  involves no filtering. 
%  note: sig1 has no preference for robustness, sig2 does.

[F1,P1]=doublex9(Astar,Bstar,C,R,Q,sig1);  % the control problem
[F2,P2]=doublex9(Astar,Bstar,C,R,Q,sig2);  % the control problem




%  Now solve the control problem when there is unobserved state. 


[F3,P3]=doublex9(Astar,Bstar,Cbar,R,Q,sig1);  % the control problem
[F4,P4]=doublex9(Astar,Bstar,Cbar,R,Q,sig2);  % the control problem


AA1=(Astar-Bstar*F1)/sqrt(delta);
AA2=(Astar-Bstar*F2)/sqrt(delta);

AA3=(Astar-Bstar*F3)/sqrt(delta);
AA4=(Astar-Bstar*F4)/sqrt(delta);

  


%  form pieces of F for purpose of constructing state space
%  for controlled system with unobserved state 


Ff3 =F3(1:nl);
Fy3=F3(nl+1:nl+ny);
Fz3=F3(nl+ny+1:nl+ny+nz);

Ff4 =F4(1:nl);
Fy4=F4(nl+1:nl+ny);
Fz4=F4(nl+ny+1:nl+ny+nz);



K=cchkz/cchky;   %  compute the ordinary Kalman gain

%  Form state space matrices for the controlled system with unobserved
%  state

AA3=[Aff-Bf*Ff3 Afy-Bf*Fy3 zeros(nl,2) -Bf*Fz3 ;
   Ayf-By*Ff3   Ayy-By*Fy3 Ayz         -By*Fz3;
   zeros(2,10)             Azz         zeros(2,2);
   zeros(2,10)            K*Ayz   Azz-K*Ayz];

AA4=[Aff-Bf*Ff4 Afy-Bf*Fy4 zeros(nl,2) -Bf*Fz4 ;
   Ayf-By*Ff4   Ayy-By*Fy4 Ayz         -By*Fz4;
   zeros(2,10)             Azz         zeros(2,2);
   zeros(2,10)            K*Ayz   Azz-K*Ayz];


BB3=[Cf;
    Cy;
    Cz;
    K*Cz];
 
 CC3=eye(14);
 DD3=zeros(14,3);
 
 sys3=ss(AA3,BB3,CC3,DD3,1);   % system with sig1, unobserved q^*
 sys4=ss(AA4,BB3,CC3,DD3,1);   % system with sig2, unobserved q^*
 
 %  The variables in sys3 and sys4 are 
 %  pi(-1) pi(-2) pi(-2) i(-1) i(-2) i(-3) q(-1) q(-2)   : i.e., f
 %  1       2       3      4     5     6     7    8 
 %  pi q  :  i.e., y
 %   9 10
 %  q^* q^*(-1)  :  i.e.,  z
 %  11  12     
 %  \check q^* \check q^*(-1)  : i.e., \check z
 %   13          14

 %  The shocks are w_pi, w_y, w_gamma, in order
 
 %  do simulation for sys3 and sys4 (filtered state)
 
x0=[zeros(9,1);.005;.005;0;.005;0]; 
  %  the initial condition is set to grow at .02 per year, with
  % the estimated initial q^* equal to q^*. 				  
  
  
  
aaaayyy=[1673;77];   % seed for random number generator, change
                     % this if you want to generate other simulations
randn('state',aaaayyy)

Tsim=100; 

u=randn(Tsim,3);
tt=0:Tsim-1;
[yy3,tt3,xx3]=lsim(sys3,u,tt,x0);
[yy4,tt4,xx4]=lsim(sys4,u,tt,x0);

hold off

figure(1)
plot(yy3(:,11),'b-')
hold on
plot(yy3(:,13),'r--'), title('q^* and hat q^*')
hold off

figure(2)
plot(yy3(:,9),'b-')
hold on
plot(100*(yy3(:,10)-yy3(:,11)),'r--'), title('\pi and g')
hold off

figure(3)
plot(yy3(:,9),'b-')
hold on
plot(100*(yy3(:,10)-yy3(:,13)),'r--'), title('\pi and hat g')
hold off

figure(4)
plot(yy3(:,10),'b-')
hold on
plot(yy3(:,11),'r--'),title('q and q^*')
hold off

figure(5)
plot(yy3(:,10),'b-')
hold on
plot(yy3(:,13),'r--')
plot(yy3(:,11),'g:'),title('q, q^*, and hat q^*')
hold off

print -deps rsfig3.eps

T=20;


figure(6)
[yh3,t1,z1]=impulse(sys3,T);
[yh4,t4,z4]=impulse(sys4,T);
%wizplot(yh1,t1,T,[1 2 3],[9 10])
wizplot2d(yh3,t1,yh4,t4,T,[1 2 3],[10 9 4]) % 4 9 and 10 are i(-1),
                    % pi and q
                    %  10 9 and 4 are q, pi, i(-1)
                    %  shocks are w_pi, w_y, w_gamma

print -deps rsfig2.eps

%-----------------------------------------------------


figure(10)
plot(yy3(:,13),'b-')
hold on
plot(yy4(:,13),'r--'), title('hat q^(0)* and hat q^*(sig2)')
hold off


figure(11)
subplot(2,2,1)
plot(yy3(:,4),'b-')
hold on
plot(yy4(:,4),'r--'),title('i (\sigma=0) and i (\sigma<0)')
hold off


%figure(12)
subplot(2,2,2)
plot(yy3(:,9),'b-')
hold on
plot(yy4(:,9),'r--'), title('\pi (\sigma=0) and \pi (\sigma<0)')

hold off

%figure(14)
subplot(2,2,3)
plot(yy3(:,10)-yy3(:,11),'b-')   %  the gap
hold on
plot (yy4(:,10)-yy4(:,11),'r--'),title('GDP gap, \sigma= 0, \sigma<0')
hold off

subplot(2,2,4)

plot(yy3(:,9),'b-')
hold on
plot(100*(yy3(:,10)-yy3(:,11)),'r--'), title('\pi and g, (\sigma=0)')
hold off

print -deps rsfig4.eps

% now simulate sys1 and sys2 (systems with observed q^*)


BB1=C;
BB2=C;

CC1=[zeros(1,9) 1 -1 0;   % pulls off g = q - q^*
     zeros(1,8) 1 0 0 0;   % pulls off pi
     0 0 0 1 zeros(1,8)];  % pulls off i
  
CC2=CC1; 
  
%    Note: the shocks are in order w_pi, w_gamma, w_y
 
 DD1=zeros(3,3);
 DD2=zeros(3,3);




%%% now let the matlab control toolkit do the work; we'll
%% do some graphing and plotting

 
   sys1=ss(AA1,BB1,CC1,DD1,1);
   sys2=ss(AA2,BB2,CC1,DD2,1);
   
   
   T = 20;
   
   


%-----------------------------------------------------

%--------------------------------------
% chao's program to compute and plot the impulse response function

[yh1,t1,z1]=impulse(sys1,T);  
[yh2,t2,z2]=impulse(sys2,T);


y1 = zeros(size(yh1,1),3,3); % here 3 is the number of observable variables and shocks we need
y2 = zeros(size(yh2,1),3,3); % here 3 is the number of observable variables and shocks we need

for i = 1:3   % pick the first three shocks
   
   %y1(:,:,i) = [yh1(:,1,i),yh1(:,3,i),yh1(:,10,i)]; % pick the observed variables, which is 
   %y2(:,:,i) = [yh2(:,1,i),yh2(:,3,i),yh2(:,10,i)]; % the 1st, 3rd and 10th column
   
   
   y1(:,:,i) = [yh1(:,1,i),yh1(:,2,i),yh1(:,3,i)]; % pick the observed variables, which is 
   y2(:,:,i) = [yh2(:,1,i),yh2(:,2,i),yh2(:,3,i)]; % the 1st, 3rd and 10th column

end

 
% y is the tensor corresponding to sys1 system. 
% If SYS has NU inputs and NY outputs, and LT=length(T), 
% where T is the simulation period, the array y is   
% LT-by-NY-by-NU  and y(:,:,j) gives 
% the impulse response of the j-th input channel.

   [rt1,ct1,pt1] = size(y1); %dimension for theta1 case
   [rt2,ct2,pt2] = size(y2); %dimension for theta2 case
   
   [rz1,cz1,pz1] = size(z1); %dimension
   [rz2,cz2,pz2] = size(z2); %dimension
   
  
% Given the particular structure of y, i.e. 51 by 3 by 3, we can easily 
% change multiple dimension array into matrix. tempts1 is a 51*3 by 3 matrix 
% obtained by stacking matrix y(:,:,j) into 51*3 by 1 column vector and 
% then make the "vectorized" y(:,:,j) become the jth column in tempts1. 

tempts1  = reshape(y1,rt1*ct1,pt1);
tempts2  = reshape(y2,rt2*ct2,pt2);

temptz1  = reshape(z1,rz1*cz1,pz1);
temptz2  = reshape(z2,rz2*cz2,pz2);

% Then we operate on tempts1 by moving "blocks" around and finally make 
% it into two new matries: capts1 and capts2, both of the dimension
% 101 by 12. 

% The first 5 columns of capts1 contains the impulse 
% responses to y from w_{1,2,3,4,5} sequentially, the second 5 columns
% contains the IMR to pi from the three shocks(6:10th), the third 5 columns 
% IMR to i(11:15)

% The  columns of capts2 contains the impulse 
% responses to the hidden states from the five shocks.
% We won't use these here (though Chao did in her program for task23)


capts1 = zeros(rt1,ct1*pt1);
capts2 = zeros(rt2,ct2*pt2);

captz1 = zeros(rz1,cz1*pz1);
captz2 = zeros(rz2,cz2*pz2);

for i = 1:ct1
   
   capts1(1:rt1,1+pt1*(i-1):pt1*i)=tempts1(1+rt1*(i-1):rt1*i,:);
   capts2(1:rt2,1+pt2*(i-1):pt2*i)=tempts2(1+rt2*(i-1):rt2*i,:);
     
end

for i = 1:cz1
   
   captz1(1:rz1,1+pz1*(i-1):pz1*i)=temptz1(1+rz1*(i-1):rz1*i,:);
   captz2(1:rz2,1+pz2*(i-1):pz2*i)=temptz2(1+rz2*(i-1):rz2*i,:);
     
end

% chao made some minor changes in figure numbers


figure(7)

for i = 1:9
      
   subplot(3,3,i) % (3,3,i): the first element: number of observable variables to be plot
                  %          the second element: number of shocks   
      
plot(t1,capts1(:,i),'r-')
hold on
plot(t2,capts2(:,i),'b--')
hold off

if i == 1
   title(['From w_{\pi}'])
elseif i == 2
   title(['From w_{y}'])
elseif i == 3
   title(['From w_{\gamma}'])
end

if i == 1
	ylabel('To g_{t}')
elseif i == 4
   ylabel('To \pi_{t}')
elseif i == 7
   ylabel('To i_{t}')
   
   %set(gca,'YAxisLocation','right')
end

axis([0 T -inf inf])

end

print -deps rsfig1.eps

return
