% Note: this program has been superceded by oligopoly2.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program computes the oligopoly example
% Robustness in forward looking models
% Stijn Van Nieuwerburgh
% First version: January 27, 2001 (stijn)
% Current version: January 27, 2001 (stijn)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% p(t)= A0-A1*(Q(t)+q(t))+v(t)
% v(t+1)= rho*v(t)+Ce*eps(t+1); eps is iid(0,1)

     A0= 10;    % intercept of demand function
     A1=1;      % slope of demand function
     rho=0.8;   % autocortrelation factor
     Ce=1 ;     % varaince of shocks
     c=1 ;      % adjustment cost factor
     b=0.95;    %discount factor
     th=10000000;     % robustness parameter

% Define the following matrices
     AA1=[1 0 0 0 0 ;0 1 0 0 0 ;0 0 1 0 0;0 0 0 1 0;A0 1 -A1 -A1 c];
     AA2=[1 0 0 0 0;0 rho 0 0 0;0 0 1 0 0;0 0 0 1 1 ;0 0 0 0 (b/c)];
     BB=[0;0;1;0;0];
     CC=[0;Ce;0;0;0];
     A=inv(AA1)*AA2;
     Bb= inv(AA1)*BB;
     C = inv(AA1)*CC;
     B=inv(AA1)*[BB CC];
     R=[c/2 0;0 -b*th] ;
     Q= [0 0 (A0/2) 0 0;0 0 .5 0 0;(A0/2) .5 -A1 -(A1/2) 0; 0 0 -(A1/2) 0 0;0 0 0 0 0];

% Define the number of 'jump variables'.
     nx=1;
     n= max(size(Q));
     nz=n-nx;

% Solve the optimal linear regulator from the above matrices
     [F P]= olrp(b,A,B,Q,R);

% Manipulate matrices

P21=P(nz+1:n,1:nz);
P22=P(nz+1:n,nz+1:n);
F1 =F(1,:) ;
F2 =F(2,:);
A1 =A(1:nz,:);
A2 =A(nz+1:n,:);
B1 =Bb(1:nz,:);
B2 =Bb(nz+1:n,:);
C1 =C(1:nz,:);
C2 =C(nz+1:n,:);

% Form the matrices M and MM

M=[eye(nz) zeros(nz,n-nz); P21 P22]*[A1-B1*F1 ;A2-B2*F1-C2*F2]*[eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)];
MM= [-inv(P22)*P21 , inv(P22)];
MM11=M(1:nz,1:nz);
MM12=M(1:nz,nz+1:n);
MM21=M(nz+1:n,1:nz);
MM22=M(nz+1:n,nz+1:n);

% Compute the ARMA representations for controls an perturbations
FF= -F*[eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)];
FF11=FF(1,1:nz);
FF12=FF(1,nz+1:n);
FF21=FF(2,1:nz);
FF22=FF(2,nz+1:n);

H1=inv(FF12'*FF12)*FF12';
H2= FF12*H1;
H3=inv(FF22'*FF22)*FF22';
H4= FF22*H3;

AR11=inv(H2)*FF12*MM22*H1;
MA11=FF11;
MA12=inv(H2)*FF12*(MM21-MM22*H1*FF11);

disp('The ARMA representation for u is:');
disp('u(t)=');disp(AR11);disp('* u(t-1) + ');disp(MA11);disp('* z(t) + ');disp(MA12); disp('* z(t-1) .');

AR21=inv(H4)*FF22*MM22*H3;
MA21=FF21;
MA22=inv(H4)*FF22*(MM21-MM22*H3*FF21);

