% oligopoly2
% Sargent's amendment of oligopoly.m, Stijn's original program
% The approximating model is computed.  See comments below.
% See oligop2.ps for description
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program computes the oligopoly example
% Robustness in forward looking models
% Stijn Van Nieuwerburgh
% First version: January 27, 2001 (stijn)
% Current version: February 16, 2001 (stijn)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% There is a representative competitive firm with cost function
%  d q_t + .5 h q_t^2 + .5 c(q_{t+1} - q_t)^2 
% A large firm has cost function
%  e Q_t + .5 g Q_t^2 + .5 c(Q_{t+1} - Q_t)^2
% Demand is 
% p(t)= ao-a1*(Q(t)+q(t))+v(t)
% v(t+1)= rho*v(t)+Ce*eps(t+1); eps is iid(0,1)
%
% Equilibrium concept: Stackelberg, with mu_{xt} being the implementation
% multiplier. 
% the state vector in the final representation is
% y_t' = [1 v_t Q_t q_t \mu_{xt}]' 
clear;

     a0= 100;     % intercept of demand function
     a1=1;        % slope of demand function
     rho=0.8;     % autocorrelation factor
     Ce=.2;       % variance of shocks
     c=1;         % adjustment cost factor
     b=0.95;      % discount factor
     d=20;        % unit cost for competitive firm
     e=20;        % unit cost for large firm
     g=.2;        %  quadratic term in cost function for large firm
     h=.2         % quadratic term in cost function for competitive firm
     
     thnr=10000000;     
     th=50;     % robustness parameter

     % Define the following matrices
     AA1=[1 0 0 0 0 ;0 1 0 0 0 ;0 0 1 0 0;0 0 0 1 0;(a0-d) 1 -a1 -a1-h c];
     AA2=[1 0 0 0 0;0 rho 0 0 0;0 0 1 0 0;0 0 0 1 1 ;0 0 0 0 (c/b)];
     BB=[0;0;1;0;0];
     CC=[0;Ce;0;0;0];
     A=inv(AA1)*AA2;
     Bb= inv(AA1)*BB;
     C = inv(AA1)*CC;
     B=inv(AA1)*[BB CC];
     
% make R and Q positive definite (in absence of be*th 
     R=[c/2 0;0 -b*th] ;  
     Q= -[0 0 ((a0-e)/2) 0 0;0 0 .5 0 0;((a0-e)/2) .5 -a1-g/2 -(a1/2) 0; 0 0 -(a1/2) 0 0;0 0 0 0 0];
               
               
     %  apparently no need to do this. taken care of in doubleo          
     % bring Q up to full rank to get detectabilitiy
     %
     % sm=.0001;
     %   sm=0;
     %  Q(2,2)=sm;  Q(4,4)=sm; Q(5,5)=sm;
     
% Define the number of 'jump variables'.
     nx=1;
     n= max(size(Q));
     nz=n-nx;

% Solve the optimal linear regulator from the above matrices
     [F P]= olrp(b,A,B,Q,R);

% Manipulate matrices

P21=P(nz+1:n,1:nz);
P22=P(nz+1:n,nz+1:n);
F1 =F(1,:) ;
F2 =F(2,:);
A1 =A(1:nz,:);
A2 =A(nz+1:n,:);
B1 =Bb(1:nz,:);
B2 =Bb(nz+1:n,:);
C1 =C(1:nz,:);
C2 =C(nz+1:n,:);

% Form the matrices M and MM

MW=[eye(nz) zeros(nz,n-nz); P21 P22]*[A1-B1*F1-C1*F2 ;A2-B2*F1-C2*F2]*[eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)];
M=[eye(nz) zeros(nz,n-nz); P21 P22]*[A1-B1*F1 ;A2-B2*F1]*[eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)];
%  MW is the worst-case law of motion, corrected from Stijn's program
%  M is the approximating law of motion.  tjs 1/30/01

MM11=M(1:nz,1:nz);
MM12=M(1:nz,nz+1:n);
MM21=M(nz+1:n,1:nz);
MM22=M(nz+1:n,nz+1:n);

% Compute the ARMA representations for controls an perturbations
FF= -F*[eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)];
FF11=FF(1,1:nz);
FF12=FF(1,nz+1:n);
FF21=FF(2,1:nz);
FF22=FF(2,nz+1:n);

AR11=FF12*MM22\FF12;
MA11=FF11;
MA12=FF12*(MM21-(MM22\FF12)*FF11);
%disp('The ARMA representation for u is:');
%disp('u(t)=');disp(AR11);disp('* u(t-1) + ');disp(MA11);disp('* z(t) + ');disp(MA12); disp('* z(t-1) .');

AR21=FF22*MM22\FF22;
MA21=FF21;
MA22=FF22*(MM21-(MM22\FF22)*FF21);
%disp('The ARMA representation for w is:');
%disp('w(t+1)=');disp(AR21);disp('* w(t) + ');disp(MA21);disp('* z(t) + ');disp(MA22); disp('* z(t-1) .');

% Following material added by tjs, 01/30/01

% first form the volatility matrix for z_t, mu_{xt}  by premultiplying

Ct=[eye(nz) zeros(nz,n-nz); P21 P22]*C;
%
%  get state space system for p, q, Q, mu_x
%
G=[a0 1 -a1 -a1 0 ;    % picks off p
   0 0 0 1 0 ;         % selects q
   0 0 1 0 0];          % selects Q
G=[G; [0 0 0 0 1]];     % selects \mu_x 
%   G=[G; FF11 FF12];   % last row picks off u=(Q_{t+1} - Q_t)

% now form the state space system M, Ct, G, zeros

DD=zeros(4,1);
sys1=ss(M,Ct,G,DD,1) ;   % system for approximating model
sys2=ss(MW,Ct,G,DD,1);   % worst case model
 
%  now compute impulse response functions for p, q, Q, u with respect
% to the scalar demand shock.  

nn=22;
y1=impulse(sys1,nn);  y1=y1(2:nn,:);
y2=impulse(sys2,nn);  y2=y2(2:nn,:);
figure(1)

tt=[0:nn-2];
subplot(4,1,1)
stairs(tt,y1(:,1)),ylabel('p')
subplot(4,1,2)
stairs(tt,y1(:,2)),ylabel('q')
subplot(4,1,3)
stairs(tt,y1(:,3)),ylabel('Q')
subplot(4,1,4)
stairs(tt,y1(:,4)),ylabel('\mu_x')

figure(2)
TT=500;
randn('seed',1000);
u=randn(TT,1);


%  construct steady state

xs=null(eye(n)-M);
xs=xs./xs(1);

ps=[a0 1 -a1 -a1  0]*xs
qs=[0 0 0 1 0]*xs
Qs=[0 0 1 0 0]*xs

xinitial=xs; xinitial(nz+1:n)=0;   % start from zero shadow price
yy1=lsim(sys1,u,[],xinitial);
subplot(3,1,1)
plot(yy1(:,2)+yy1(:,3),'-g'),ylabel('q+Q')
subplot(3,1,2),
plot(yy1(:,2),'-b'),ylabel('q')
subplot(3,1,3)
plot(yy1(:,3),'--r'),ylabel('Q')

figure(3)
subplot(3,1,1)
plot(yy1(:,4),'-g'),ylabel('\mu_x')
subplot(3,1,2)
plot(yy1(:,3),'-b'),ylabel('Q')
subplot(3,1,3)
plot(yy1(:,1),'--r'),ylabel('p')

var(yy1(:,2)+yy1(:,3))