function[Kr,Kphi,Sig,DelSig]=rfilter(A,C,G,D,H,bet,thet)
% function[Kr,Sig,DelSig]=rfilter(A,C,G,D,H,bet,thet) 
%  solves the discounted filtering problem for the state space system
%
%  x_{-t} = A x_{-t-1} + C (\epsilon_{-t} + w_{-t})
%
%  y_{-t} = G x_{-t-1} + D (\epsilon_{-t} + w_{-t})
%
% and the objective function for the zero-sum two player game is
%
% - .5 \sum_{t=0}^\infty \beta^t [z_{-t}' z_{-t} - \thet w_{-t}' w_{-t}] 
% 
% where z_{-t} = H x_{-t}. 
%
% The robust filter has the form
%
%  \hat x_{-t} = A \hat x_{-t-1} _ Kr [y_t - \hat y_{-t-1}]
%
% The program returns the robust Kalman gain Kr as well Sig, the covariance
% matrix of the state estimate and DelSig, which is {\cal D}^*(\Sig).  See
% HS first chapter on robust filtering.  
%
% The program transforms the problem into the dual discounted linear regulator
% problem described in HS chapter on dual filtering, using tricks to
% transform  a discounted problem with cross products between states into controls
% into one without cross products and without discounting.  This allows us
% to use doublex9.m (the doubling algorithm for the linear regulator) to
% solve the problem.  



%  setup the dual linear regulator
Q=C*C';
R=D*D';
W=C*D';  
AA=bet^(-1)*A';
BB=bet^(-1)*G';
DD=H';   % this is D in doublex9, relabelled 

% Now use "trick" to eliminate discount rate and cross-products between
% states and controls.  (NOTE: for many problems, W = 0 and this step can
% be skipped. In this case, just set As=sqrt(beta)*AA, Bs=sqrt(beta)*BB;
% add a loop to make this optional)




[As,Bs,Qs]=trick(bet,AA,BB,Q,R,W);

%% now use doublex9 to solve the dual control problem
% first set sig (the risk-sensitivity parameter)
%
sig=-thet^(-1);


[F,K,P,Pt]=doublex9(As,Bs,DD,Qs,R,sig);




% readjust for the cross-product term
Kphi=K+R\W';
Kr=F+R\W';
Sig=P;
DelSig=Pt;

DelSig1=Sig +thet^(-1)*Sig*H'*inv(thet-H*Sig*H')*H*Sig;