%  robust_fringe_1.m  
%  July 18-19, 2006
%  new version of compfringe.m
%  
%  This version works.
%  take account of the fact that doublex9 asks for a MINIMUM problem and therefore treats the second problem appropriately.
%  Please notice the SIGNS on the Q and R. 
% 
%  We have to remember that
% we are solving a maximum problem and that doublex9 is set up for a
% minimum problem.  A negative sigma in doublex9 corresponds to more
% robustness for the minimum problem. 
%
% There is a representative competitive firm with cost function
%  d q_t + .5 h q_t^2 + .5 c(q_{t+1} - q_t)^2 
% A large firm has cost function
%  eQ_t + .5 g Q_t^2 + .5 c(Q_{t+1} - Q_t)^2
% Demand is 
% p(t)= ao-a1*(Q(t)+q(t))+v(t)
% v(t+1)= rho*v(t)+Ce*eps(t+1); eps is iid(0,1)
%
% Equilibrium concept: Stackelberg, with mu_{xt} being the implementation
% multiplier. 
% the state vector in the final representation is
% y_t' = [1 v_t Q_t \bar q_t \mu_{qt} mu_{vt}]' (6 \times 1)

     a0= 100;    % intercept of demand function
     a1=1;      % slope of demand function
     rho=0.8;   % autocorrelation factor  % originally set at .8
     Ce=.2;     % variance of shocks
     c=10;      % adjustment cost factor  % originally set at 1
     b=0.95;    % discount factor
     
     d=20;      % unit cost for competitive firm
     e=20;      % unit cost for large firm
     g=1;     %  quadratic term in cost function for large firm; originally set at .2
     h=1;      % quadratic term in cost function for competitive firm% originally set at .2
     
     thm=10;   % theta for monopolist
     thf=30;   % theta for fringe

 % please note that because we are using doublex9, which solves robust minimum problems, we have to be 
 % careful about the signs of Q,R, and sig. Negative sigmas mean more
 % robustness.  
     
  
% set up the grand loop over the sigv, sigq
 % set initial conditions
 
 sigv_2=.0100;
    sigq_2=.0100;
 sigv=sigv_2;
    sigq=sigq_2;
 % set relaxation parameter
 grelax=1;   % .01 works fine
 sigdiff=[100;100];
%for i=1:10000;
ctol=.0000000000001
i=0;
while (norm(sigdiff)>ctol) 
    i=i+1;
    sigq= (1-grelax)*sigq+ grelax*sigq_2;
 sigv=(1-grelax)*sigv+ grelax*sigv_2;



% Define the following matrices
     AA1=eye(6);
     AA2=[1 0 0  0 0  0;
         0 rho  0 0 0 0;
         0 0 1 0 0 0;
         c^(-1)*(d-a0) -c^(-1) c^(-1)*a1 c^(-1)*(a1+h)+1 c^(-1)*b^(-1) 0;
         (-a0+d) -1 a1 (a1+h-(sigq*Ce/(2*b*thf*rho))) b^(-1) (sigq*Ce/(2*b^2*thf*rho));
         0 0 0 -rho^(-1)*(1 + (sigv*Ce/(2*b*thf))) 0 ((b*rho)^(-1)+(sigv*Ce/(2*b^2*thf*rho))) ]
 
         
        
     BB=[0;0;1;0;0;0];  % these two matrices are set up in case we want to trick olrp.m to solve the max min problem.  See compfringe.m.
     CC=[0; Ce;0;0; sigq; sigv];
     
     A=inv(AA1)*AA2;
     Bb= inv(AA1)*BB;
     C = inv(AA1)*CC;
     B=inv(AA1)*[BB CC];
   
% please note the "signs" of Q and R -- this is pertinent when we set up
% doublex9 and flip to a minimum problem.
     Q= -[0 0  ((a0-e)/2) 0 0 0;
         0 0  .5 0 0 0;
         ((a0-e)/2)  .5 -a1-g/2 -(a1/2) 0 0;
         0 0 -(a1/2) 0 0 0;
         0 0 0  0 0 0 ;
         0 0 0 0 0 0 ];
     Rs=c/2;
    
     
% Define the number of 'jump variables'.

     nx=2;
     n= max(size(Q));
     nz=n-nx;


 
  As=A*sqrt(b)
  Bs=Bb*sqrt(b)

  %  Q and R are not affected 
  
  Cs=C;
  
  % Note: the problem is to max min \sum \beta^t { - y' Q y - U' R U ... }
  % where Q, R as defined in the chapter,
  % which is equivalent with min max \sum \beta^t { y' Q y + U' R U ....},
  % adjusting thm as indicated.  Remember that doublex9 solves a min
  % problem.
  
  [Fs,Ks,Ps,Pst]=doublex9(As,Bs,Cs,Q,Rs,-1/thm);  % flip sign of sig because we have a max problem
  Ks=b^(-.5)*Ks;
  
    
  Fnew(2,:)=-Ks;
  Fnew(1,:)=Fs;
  
  F=Fnew;
  P=Ps;  

 
  

% Manipulate matrices

P21=P(nz+1:n,1:nz);
P22=P(nz+1:n,nz+1:n);
F1 =F(1,:) ;
F2 =F(2,:);
A1 =A(1:nz,:);
A2 =A(nz+1:n,:);
B1 =Bb(1:nz,:);
B2 =Bb(nz+1:n,:);
C1 =C(1:nz,:);
C2 =C(nz+1:n,:);



%%%======================================

%disp('Law of motion cal M for z, mu_x = '),
 
M=[eye(nz) zeros(nz,n-nz); P21 P22]*[A1-B1*F1 ;A2-B2*F1]*[eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)];
% MW=[eye(nz) zeros(nz,n-nz); P21 P22]*[A1-B1*F1-C1*F2 ;A2-B2*F1-C2*F2]*[eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)];  % this is from
%    % the old compfringe.m program -- not how it builds in the worst case
%    % shock
%    
% M=MW;  % temporary check
CM0=C;
%CM1= [eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)]*CM0;  %this is the
%original, which seems wrong -- see \Ep{CMdef} in the monograph
CM1= [eye(nz) zeros(nz,n-nz); P21 P22]*CM0;

%  M, CM  is the law of motion of z_t, \mu_{xt}
CM=CM1;

CM2=CM(nz+1:n,1);

% Now to prepare the recursive version of the decision problem of the
% competitive fringe firm, transform the state to z_t, \mu_{xt}-CM2 v_t.
% This transforms the state space to make v be the only component of the
% exogenous state being hit by the shock. 


M11=M(1:nz,1:nz);
M12=M(1:nz,nz+1:n);
M21=M(nz+1:n,1:nz);
M22=M(nz+1:n,nz+1:n);


A11b=M11+M12*CM2*[0 1 0 0];
A12b=M12;
A21b=M21+(M22-rho*eye(2))*CM2*[0 1 0 0];
A22b=M22;


Ab=[A11b A12b zeros(nz,1);
    A21b A22b  zeros(n-nz,1);
    zeros(1,n) 1];
Af=Ab;

% The state for the competitive firm is now  z_t,  \mu_{xt}-CM2 v_t, q_t.

% set up the robust linear regulator for the competitive fringe firm.



Qf=[(a0-d) 0 0 0 0 0 0;
    0 0 0 0 0 0 .5;
    0 0 0 0 0 0 -a1/2;
    0 0 0 0 0 0 -a1/2;
    0 0 0 0 0 0 0;
    0 0 0 0 0 0 0;
    0 .5 -a1/2 -a1/2 0 0 -.5*h];

Rf=-c/2;

% Note the sign convention -- which is the opposite that we used for the
% monopolist (for no good reason) -- this means that we'll have to flip
% signs of Qf, Rf when we pose the minimum problem solved by doublex9.


Bf=[0 0 0 0 0 0 1]';
Cf=[0 Ce 0 0 0 0 0]';








% This is an old shell -- can be adapted to trick olrp to solve the robust
% control problem.  We checked that this gives the same answer in
% compfringe.m and so discard it here.
% 
% % Solve the optimal linear regulator from the above matrices
% RMM=[-c/2 0; 0 -b*thf];
% % BM= [zeros(8,1);1];
% % CM=[CM1;Ce;0];
% BMM=[BM CM];
% esmall=.0000001;
% QMM=QM+eye(9)*esmall;
% tic;     
% [Fo Po]= olrp(b,AM,BMM,QMM,RMM);
% toc;




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% Now solve the representative firm's decision problem:
% Use the trick to adjust for discounting

Ms=Af*sqrt(b)
  Bs=Bf*sqrt(b)
 
 
  [FM,KM,PM,PMt]=doublex9(Ms,Bs,Cf,-Qf,-Rf,-1/thf);   
  
 % Tom: double check the following two command lines:
  
  KM=b^(-.5)*KM;
  
 % adjust the sign on F because of the +, - convention for the K, M
 
   FM=-FM; 
  
 % now compute the sigma_v, sigma_q on the multipliers
 
 sigall= -2*PM*Cf;
 
 sigq_2=sigall(6);
 sigv_2=sigall(7);
 
 sigdiff=[sigq-sigq_2;sigv-sigv_2];
 
end  % this is the end of the sigma loop




% now compute key objects and do checks

% (1) the formula for x_t = [lmabda^q, \lambda^v]:
% coefficients on [z_t \mu_{xt}]

coefflambda=[ - P22\P21 inv(P22)] 

% (2) law of motion for [z_{t+1} \mu_{xt}]:

M
CM

% first formula for E_t \lambda_{t+1}^v:
% = (beta *\rho)^(-1) \lambda_t^v - \rho^{-1} q_t

coefflamv= (b*rho)^(-1)*coefflambda(2,:);
coefflamv(4)=coefflamv(4)-rho^(-1);

% now formula for E_t \lambda_{t+1}^q
% = beta^(-1) \lambda_t^q -[a0 - a1(Q_t + \bar q_t) + v_t ] + d + h \bar q_t 

coefflamq= b^(-1)*coefflambda(1,:);
coefflamq(1)=coefflamq(1)+d-a0; % constant
coefflamq(2)=coefflamq(2) - 1;  % v
coefflamq(3)=coefflamq(3)+ a1  ; %  Q
coefflamq(4)=coefflamq(4)+ a1+ h;  % \bar q 


%%  
% now get first formulas for u_t, w_{t+1}
%
% first, u_t = c^(-1) E_t \lambda_{t+1}^q:

coeffu = c^(-1)*coefflamq

% then, w_{t+1} = -{C_e/2* beta*thf} E_t \lambda_{t+1}^v:

coeffw= -(Ce/(2*b*thf))*coefflamv  


%% now do the comparison from the recursive representation of the problem of the competitive fringe:

% compute the matrix that transforms [z_t \mu_{xt}] to X_t:
% i.e., X_t = Tx [z_t \cr \mu_{xt}]
% where X_t = [1 v_t Q_t \bar q_t (\mu_{xt}- CM2 v_t) \bar q_t],
% z_t = [1 v_t Q_t \bar q_t]

Tx=[1 0 0 0 zeros(1,2);
    0 1 0 0 zeros(1,2);
    0 0 1 0 zeros(1,2);
    0 0 0 1 zeros(1,2);
    zeros(2,1) -CM2 zeros(2,1) zeros(2,1) eye(2);
    0 0 0 1 zeros(1,2);]


% Now check the coefficients on u and w;

coeffu_check = FM*Tx
coeffu


coeffu./coeffu_check

coeffw_check=KM*Tx
coeffw


coeffw./coeffw_check

% now compute E_t \lambda_{t+1}

Aff=Af-Bf*FM;  % altered after 3RA's correction

coefflambda_check=-2*PM*Aff*Tx
coefflambda




 
 
 return
  
  
 