%  robust_monop_fringe_1.m  
%  August 26, 2006
%  
%  
%  
%  take account of the fact that doublex9 asks for a MINIMUM problem and therefore treats the second problem appropriately.
%  Please notice the SIGNS on the Q and R. 
% 
%  We have to remember that
% we are solving a maximum problem and that doublex9 is set up for a
% minimum problem.  A negative sigma in doublex9 corresponds to more
% robustness for the minimum problem. 
%
% There is a representative competitive firm with cost function
%  d q_t + .5 h q_t^2 + .5 c(q_{t+1} - q_t)^2 
% A large firm has cost function
%  eQ_t + .5 g Q_t^2 + .5 c(Q_{t+1} - Q_t)^2
% Demand is 
% p(t)= ao-a1*(Q(t)+q(t))+v(t)
% v(t+1)= rho*v(t)+Ce*eps(t+1); eps is iid(0,1)
%
% Equilibrium concept: Stackelberg, with mu_{xt} being the implementation
% multiplier. 
% the state vector in the final representation is
% y_t' = [1 v_t Q_t \bar q_t \mu_{qt} mu_{vt}]' (6 \times 1)

     a0= 100;    % intercept of demand function
     a1=1;      % slope of demand function
     rho=0.9;   % autocorrelation factor  % originally set at .8
     Ce=2;     % variance of shocks
     c=10;      % adjustment cost factor  % originally set at 1
     b=.95;    % discount factor
     
     d=20;      % unit cost for competitive firm
     e=20;      % unit cost for large firm
     g=1;     %  quadratic term in cost function for large firm; originally set at .2
     h=1;      % quadratic term in cost function for competitive firm% originally set at .2
     
     thm=20000000000;   % theta for monopolist
     thf=30000000000;   % theta for fringe     
     
          

 % please note that because we are using doublex9, which solves robust minimum problems, we have to be 
 % careful about the signs of Q,R, and sig. Negative sigmas mean more
 % robustness.  
     
  
% set up the grand loop over the sigv, sigq, sigQ, sigbarq and the H and L
% coefficients
 % set initial conditions
 
 
 sigv_2= 0.01;
 sigq_2= 0.01;
 sigQ_2=.01;
 sigbarq_2=.01;
 HV_2=.01;
 LV_2=.01;
 HQ_2=.01;
 LQ_2=.01;
 Hbarq_2=.01;
 Lbarq_2=.01;
 
 
 sigQ=sigQ_2;
 sigbarq=sigbarq_2;
 sigv=sigv_2;
 sigq=sigq_2;
 HV=.0;
 LV=.0;
 HQ=.0;
 LQ=.0;
 Hbarq=.0;
 Lbarq=.0;

 % set relaxation parameter
 grelax=.1;   
 pardiff=100*ones(10,1);
 par=[sigv;sigq;sigQ;sigbarq;HV;LV;HQ;LQ;Hbarq;Lbarq];
 par2=zeros(10,1);
 
 
par2(1)=sigv_2
par2(2)=sigq_2;
par2(3)=sigQ_2;
par2(4)=sigbarq_2;
par2(5)=HV_2;
par2(6)=LV_2;
par2(7)=HQ_2;
par2(8)=LQ_2;
par2(9)=Hbarq_2;
par2(10)=Lbarq_2;

 %  Here is the loop for computing the equilibrium sigma_v, \sigma_q, etc.
%ctol=1e-12;
ctol=1e-8;
ii=0;
while (norm(pardiff)>ctol) 
    ii=ii+1;
    par =(1-grelax)*par + grelax*par2;  
sigv=par(1);
sigq=par(2);
sigQ=par(3);
sigbarq=par(4);
HV=par(5);
LV=par(6);
HQ=par(7);
LQ=par(8);
Hbarq=par(9);
Lbarq=par(10);

%% now define the elements of the D matrices

D1=[b*rho b*HV b*LV;
    0 b*(1+HQ) b*LQ;
    0 b*Hbarq b*(1+Lbarq)];

D2=[1 0 0 -1;
    0 1 0 a1;
    0 0 1 a1];

DD=D1\D2;

sigt=[sigv;sigQ; sigbarq];

alpha=-(Ce/(2*thf))*DD(1,:);

DN=DD-sigt*alpha;




% Define the following matrices
cbi=1/c;    
bi=b^(-1);
     AA1=eye(8);
     
     AA2=zeros(8);
     
     
%      AA2(6,6)=DN(1,1);  % lambda_v on lambda_v
%      AA2(6,7)=DN(1,2);  % lambda_v on lambda_Q
%      AA2(6,8)=DN(1,3);  % lambda_v on lambda_barq
%      AA2(6,4)=DN(1,4);  % lambda_v on q
%           
%      AA2(7,6)=DN(2,1);  %lambda_Q on lamba_v
%      AA2(7,7)=DN(2,2);  %lambda_Q on lambda_Q
%      AA2(7,8)=DN(2,3);  %lambda_Q on lambda_barq
%      AA2(7,4)=DN(2,4);  %lambda_barq on q
%      
%      AA2(8,6)=DN(3,1);  %lambda_barq on lambda_v
%      AA2(8,7)=DN(3,2);  %lambda_barq on lambda_Q
%      AA2(8,8)=DN(3,3);  %lambda_barq on lambda_barq
%      AA2(8,4)=DN(3,4);  %lambda_barq on q
     
    AA2(6:8,6:8)=DN(:,1:3);   %lambda_v, lambda_Q, lambda_barq on lambda_v, lambda_Q, lambda_barq
    AA2(6:8,4)=DN(:,4);     %lambda_v, lambda_Q, lambda_barq on \barq
%   These are the same as what was done longhand just above
      
     
     
  AA2(5,:)=[bi*(d-a0) -bi bi*a1 bi*(a1+h) bi 0 0 0];  %lambda_q
  
  AA2(5,6:8)= -sigq*alpha(1:3); % lambda_q on lambda_v, lambda_Q, lambda_barq
  AA2(5,4)=AA2(5,4)-sigq*alpha(4);  %lambda_q on \barq
     

  
  AA2(1,:)=[1 zeros(1,7)];   % 1
  
  AA2(2,:)=[0 rho zeros(1,6)];  % v
  
  AA2(3,:)=[0 0  1 zeros(1,5)];   % Q
  
  AA2(4,:)=c^(-1)*b*AA2(5,:);  % \bar q 
  AA2(4,4)=AA2(4,4)+1;   % bar q on barq 
  
 
  
         
        
     BB=[0;0;1;0;0;0;0;0];  
     CC=[0; Ce;0;0; sigq; sigv; sigQ; sigbarq];
     
     A=inv(AA1)*AA2;
     Bb= inv(AA1)*BB;
     C = inv(AA1)*CC;
     B=inv(AA1)*[BB CC];
     
    
   
% please note the "signs" of Q and R -- this is pertinent when we set up
% doublex9 and flip to a minimum problem.
     Q= -[0 0  ((a0-e)/2) 0 0 0 0 0;
         0 0  .5 0 0 0 0 0;
         ((a0-e)/2)  .5 -a1-g/2 -(a1/2) 0 0 0 0;
         0 0 -(a1/2) 0 0 0 0 0;
         0 0 0  0 0 0 0 0 ;
         0 0 0 0 0 0 0 0;
         zeros(2,8)];
     Rs=c/2;
    
     
% Define the number of 'jump variables'.

     nx=4;
     n= max(size(Q));
     nz=n-nx;


 
  As=A*sqrt(b);
  Bs=Bb*sqrt(b);

  %  Q and R are not affected 
  
  Cs=C;
  
  % Note: the problem is to max min \sum \beta^t { - y' Q y - U' R U ... }
  % where Q, R as defined in the chapter,
  % which is equivalent with min max \sum \beta^t { y' Q y + U' R U ....},
  % adjusting thm as indicated.  Remember that doublex9 solves a min
  % problem.
  
  % flip sign of sig because we have a max problem
  [Fs,Ks,Ps,Pst]=doublex9(As,Bs,Cs,Q,Rs,-1/thm);
  Ks=b^(-.5)*Ks;
   
  Fnew(2,:)=Ks;
  Fnew(1,:)=Fs;
     
  F=Fnew;
  P=-Ps;
% Manipulate matrices

P21=P(nz+1:n,1:nz);
P22=P(nz+1:n,nz+1:n);


 % partition C to prepare for computing  new values of sigq, sigv, sigQ,
 % sigbarq
 
 Cz=C(1:4);    % these are the volatility terms on the natural state z
 
 Cx = -P22\P21*Cz;  % these are the implied volatility terms on the multipliers x
 

 
 
 sigq_2=Cx(1);
 sigv_2=Cx(2);
 sigQ_2=Cx(3);
 sigbarq_2=Cx(4);        % note the order of these objects in Cx
 
 
 
 % Now compute H and L, the rules for U and u:
 
 
% Monopolist's rule for U, W is

Mrule=[-Fs ; Ks]*[eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)] ;

H=Mrule(1,:);

% Now get the rule for u


% first, get the multipliers:

coefflambda=[ - P22\P21 inv(P22)] ;


%  formula for E_t \lambda_{t+1}^q
% = beta^(-1) (\lambda_t^q -[a0 - a1(Q_t + \bar q_t) + v_t ] + d + h \bar
% q_t )

coefflamq= b^(-1)*coefflambda(1,:);
coefflamq(1)=coefflamq(1)+bi*(d-a0); % constant
coefflamq(2)=coefflamq(2) - bi;  % v
coefflamq(3)=coefflamq(3)+ bi*a1  ; %  Q
coefflamq(4)=coefflamq(4)+ bi*(a1+ h);  % \bar q 


%  u_t = b*c^(-1) E_t \lambda_{t+1}^q = L*state:

L = b*c^(-1)*coefflamq ;  % rule for u


% Pick off the coefficients on v, Q, \bar q in the H and L rules for U and \bar u: 

HV_2=H(2);
HQ_2=H(3);
Hbarq_2=H(4);

LV_2=L(2);
LQ_2=L(3);
Lbarq_2=L(4);


% list the revised parameters:

par2(1)=sigv_2;
par2(2)=sigq_2;
par2(3)=sigQ_2;
par2(4)=sigbarq_2;
par2(5)=HV_2;
par2(6)=LV_2;
par2(7)=HQ_2;
par2(8)=LQ_2;
par2(9)=Hbarq_2;
par2(10)=Lbarq_2;

% compute the proposed (full) revision of the parameter vector
pardiff=par-par2;
 

 
end  % this is the end of the loop searching over the sigma's and the H and L coefficients



%% Now compute objects that characterize the law of motion under the
%% monopolist's robust rule and the approximating model as well as the
%% monopolist's choice of U, W as a function of z, \mu_x:


F1 =F(1,:) ;
F2 =F(2,:);
A1 =A(1:nz,:);
A2 =A(nz+1:n,:);
B1 =Bb(1:nz,:);
B2 =Bb(nz+1:n,:);
C1 =C(1:nz,:);
C2 =C(nz+1:n,:);

%%%======================================

%disp('Law of motion cal M for z, mu_x = '),
 
M=[eye(nz) zeros(nz,n-nz); P21 P22]*[A1-B1*F1 ;A2-B2*F1]*[eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)];


CM0=C;

CM1= [eye(nz) zeros(nz,n-nz); P21 P22]*CM0;

%  M, CM  is the law of motion of z_t, \mu_{xt}
CM=CM1;

CM2=CM(nz+1:n,1);   % CM2 should equal 0 for the following reason:

 %  the state y=[z x] in the monopolist's problem, where x are the
 %  lambda's.  The volatility loadings are Cz, Cx where Cx are the loadings
 %  on the lambda's.  Because the volatility loadings on \mu_x should be
 %  zero due to the measurability requirements for them, we should have
 %  Cx=-P22^(-1)*P21 Cz  or P21 Cz + P22 Cx = 0, which implies that the
 %  last block of rows of CM1 should be zero (see above equation).
 %
 %  We seem to have numerical problems with this if we set the tolerance
 %  ctol too low.


% Monopolist's rule for U, W is
disp('monopolist rule for U, W')
Mrule=[-Fs ; Ks]*[eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)]

% Monopolist's choice of ubar rule is
disp('monopolist rule for u')
L


% Now to prepare the recursive version of the decision problem of the
% competitive fringe firm, transform the state to z_t, \mu_{xt}-CM2*Ce^{-1} v_t.
% This transforms the state space to make v be the only component of the
% exogenous state being hit by the shock. 

M11=M(1:nz,1:nz);
M12=M(1:nz,nz+1:n);
M21=M(nz+1:n,1:nz);
M22=M(nz+1:n,nz+1:n);


A11b=M11;  
A12b=M12;
A21b=M21;  %corrected by Tom, August 2, 2006
A22b=M22;


Af=[A11b A12b zeros(nz,1);
    A21b A22b  zeros(n-nz,1);
    zeros(1,n) 1];


% The state for the competitive firm is now  z_t,  \mu_{xt}, q_t.

% set up the robust linear regulator for the competitive fringe firm.


Qf=[0 0 0 0 0 0 0 0 (a0-d)/2;
    0 0 0 0 0 0 0 0 .5;
    0 0 0 0 0 0 0 0 -a1/2;
    0 0 0 0 0 0 0 0 -a1/2;
    0 0 0 0 0 0 0 0 0;
    0 0 0 0 0 0 0 0 0;
    0 0 0 0 0 0 0 0 0;
    0 0 0 0 0 0 0 0 0;
    (a0-d)/2 .5 -a1/2 -a1/2 0 0 0 0 -.5*h];
    
Rf=-c/2;

% Note the sign convention -- which is the opposite that we used for the
% monopolist (for no good reason) -- this means that we'll have to flip
% signs of Qf, Rf when we pose the minimum problem solved by doublex9.


Bf=[0 0 0 0 0 0 0 0 1]';
Cf=[0 Ce 0 0 0 0 0 0 0]';
CMM=[CM;0];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% Now solve the representative firm's decision problem:
% Use the trick to adjust for discounting

Ms=Af*sqrt(b);
  Bs=Bf*sqrt(b);
 
 
  [FM,KM,PM,PMt]=doublex9(Ms,Bs,Cf,-Qf,-Rf,-1/thf);  

 % Tom: double check the following two command lines:
  
  KM=b^(-.5)*KM;
  
  PM=-PM;
  
 % now compute the sigma_v, sigma_q, sigma_Q, sigma_barq volatilities on the multipliers using the
 % value function for the competitive firm
 
 sigall= 2*PM*Cf;
 
 sigv_cm=sigall(2);    %  multiplier on v 
 sigq_cm=sigall(9);    %  multipler on q  
 sigQ_cm=sigall(3);
 sigbarq_cm=sigall(4);
 
 

 
 %   The following should match
 
  
 sigstack=[sigv;sigq;sigQ;sigbarq];   % sig volatility loadings from monopolist's problem
 
 sigcomp=[sigv_cm;sigq_cm;sigQ_cm;sigbarq_cm];  % sig volatility loading from the competitive firm's recursive problem
 
 disp('sigstack')
 sigstack
 disp('sigcomp')
 sigcomp
 
 disp('sigstack./sigcomp')
 sigstack./sigcomp
 
 
 

% 


% matrix that converts the "long state" in the fringe's  problem to the
% short one in the monopolist's problem

Tx=[eye(8) zeros(8,1)];
Tx(4,9)=1;   % Adds coefficient on q to coefficient on \barq

Tx




% now compute key objects and do checks


% Here we compute the formula for the multipliers two ways, first from the
% monopolist's problem, then from the Bellman equation for the competitive
% fringe. 

% First, we'll use the monopolist's problem:
%  (1) the formula for x_t = [lmabda^q, \lambda^v]:
%   coefficients on [z_t \mu_{xt}]

coefflambda=[ - P22\P21 inv(P22)] 

lamshort=[ - P22\P21 inv(P22)] 

% now use the Bellman equation for the competitive fringe

lamlong0=2*PM;



lamlong=[lamlong0(9,:);   % multiplier on q in competitive firm's problem
         lamlong0(2,:);   % multiplier on v in competitive firm's problem
         lamlong0(3,:);   % multiplier on Q in competitive firm's problem
         lamlong0(4,:)];  % multiplier on barq in competitive firm's problem

  %note: the state in lamlong is 1,v,Q,barq,mu_z,q
  % to compare to lamshort, we have to add the coefficients on q and \bar
  % q.  The following two lines do this.
   
lamlongfinal=lamlong(:,1:8);
lamlongfinal(:,4)=lamlongfinal(:,4)+lamlong(:,9);



disp('lamshort')
lamshort
disp('lamlongfinal')
lamlongfinal


%  disp('lamshort/lamlong')
 lamshort./lamlongfinal
 

 




% Next, we move on to compute the decision rules for u and w for the
% competitive fringe in two ways, one from the monopolist's problem, the
% other from the competitive fringe's Bellman equation


% (2) law of motion for [z_{t+1} \mu_{xt}]:

M
CM

% first formula for E_t \lambda_{t+1}^v:
% = (beta *\rho)^(-1) (\lambda_t^v -  q_t)

coefflamv= (b*rho)^(-1)*coefflambda(2,:);
coefflamv(4)=coefflamv(4)-rho^(-1)/b;

% now formula for E_t \lambda_{t+1}^q
% = beta^(-1) (\lambda_t^q -[a0 - a1(Q_t + \bar q_t) + v_t ] + d + h \bar
% q_t )

coefflamq= b^(-1)*coefflambda(1,:);
coefflamq(1)=coefflamq(1)+bi*(d-a0); % constant
coefflamq(2)=coefflamq(2) - bi;  % v
coefflamq(3)=coefflamq(3)+ bi*a1  ; %  Q
coefflamq(4)=coefflamq(4)+ bi*(a1+ h);  % \bar q 


%%  
% now get first formulas for u_t, w_{t+1}
%
% first, u_t = b*c^(-1) E_t \lambda_{t+1}^q:

coeffu = b*c^(-1)*coefflamq   % compare this with formula in paper

% then, w_{t+1} = -{C_e/2* beta*thf} E_t \lambda_{t+1}^v:

coeffw= -(Ce/(2*thf))*coefflamv  



% Now compute the formulas for decision rules for  u and w from the competitive fringe's Bellman equation;

% first try for  u rule




coeffu_check = -FM
coeffu



%coeffu./coeffu_check


% now try for w rule

coeffw_check=KM
coeffw


%coeffw./coeffw_check

return




% now compare two ways of computing E_t \lambda_{t+1}

Aff=Af-Bf*FM;  

coefflambda_check=2*PM*Aff;

Etlambda_check=[coefflambda_check(7,:);  % picks off q 
    coefflambda_check(2,:)]; % picks off v


Etlambda=[coefflamq;coefflamv]  % It seems better for q than for v -- I have to figure out how they should match up


Etlambda_check

Etlambda
 


disp('assemble checks')

Etlambda

Etlambda_check
 


coeffw./coeffw_check

coeffu./coeffu_check

 sigv/sigv_c2
  
 
 sigq/sigq_c2
 
lamlong./lamshort
 
 return
  
  
