function[ps,qs,Qs,muqs,Ws]=robust_stackelbergfn(thm);
% function[ps,qs,Qs,muqs,Ws]=robust_stackelbergfn(thm)  
% this program solves the robust-Stackelberg problem for the parameters
% locked in below for different values of thm, then gives steady state
% values. 
%
% There is a representative competitive firm with cost function
%  d q_t + .5 h q_t^2 + .5 c(q_{t+1} - q_t)^2 
% A large firm has cost function
%  eQ_t + .5 g Q_t^2 + .5 c(Q_{t+1} - Q_t)^2
% Demand is 
% p(t)= ao-a1*(Q(t)+q(t))+v(t)
% v(t+1)= rho*v(t)+Ce*eps(t+1); eps is iid(0,1)
%
% Equilibrium concept: Stackelberg, with mu_{xt} being the implementation
% multiplier. 
% the state vector in the final representation is
% y_t' = [1 v_t Q_t \bar q_t \mu_{qt} mu_{vt}]' (6 \times 1)

     a0= 100;    % intercept of demand function
     a1=1;      % slope of demand function
     rho=0.8;   % autocorrelation factor  % originally set at .8
     Ce=2;     % variance of shocks
     c=10;      % adjustment cost factor  % originally set at 1
     b=.95;    % discount factor
     
     d=20;      % unit cost for competitive firm
     e=20;      % unit cost for large firm
     g=1;     %  quadratic term in cost function for large firm; originally set at .2
     h=1;      % quadratic term in cost function for competitive firm% originally set at .2
     
     %  thm=20;   % theta for monopolist
    
       
     
    

 % please note that because we are using doublex9, which solves robust minimum problems, we have to be 
 % careful about the signs of Q,R, and sig. Negative sigmas mean more
 % robustness.  
     
  
% set up the grand loop over the sigv, sigq
 % set initial conditions
 
 

 sigq_2= 0.01;
 
 sigq=sigq_2;

 % set relaxation parameter
 grelax=0.1;   
 sigdiff=100;

 
 %  Here is the loop for computing the equilibrium sigma_v, \sigma_q
ctol=1e-12;
i=0;
while (norm(sigdiff)>ctol) 
    i=i+1;
    sigq= (1-grelax)*sigq+ grelax*sigq_2;
 


% Define the following matrices
cbi=1/c;    % this is a key change that corrects an error
bi=b^(-1);
     AA1=eye(5);
     AA2=[1 0 0  0 0  ;
         0 rho  0 0 0 ;
         0 0 1 0 0 ;
         cbi*(d-a0) -cbi cbi*a1  cbi*(a1+h)+1 cbi ;
         bi*(-a0+d) -bi bi*a1 (a1+h)*bi bi] ;
         
 
         
        
     BB=[0;0;1;0;0];  % these two matrices are set up in case we want to trick olrp.m to solve the max min problem.  See compfringe.m.
     CC=[0; Ce;0;0; sigq];
     
     A=inv(AA1)*AA2;
     Bb= inv(AA1)*BB;
     C = inv(AA1)*CC;
     B=inv(AA1)*[BB CC];
     
    
   
% please note the "signs" of Q and R -- this is pertinent when we set up
% doublex9 and flip to a minimum problem.
     Q= -[0 0  ((a0-e)/2) 0 0 ;
         0 0  .5 0 0 ;
         ((a0-e)/2)  .5 -a1-g/2 -(a1/2) 0 ;
         0 0 -(a1/2) 0 0 ;
         0 0 0  0 0 ] ;
        
     Rs=c/2;
    
     
% Define the number of 'jump variables'.

     nx=1;
     n= max(size(Q));
     nz=n-nx;


 
  As=A*sqrt(b);
  Bs=Bb*sqrt(b);

  %  Q and R are not affected 
  
  Cs=C;
  
  % Note: the problem is to max min \sum \beta^t { - y' Q y - U' R U ... }
  % where Q, R as defined in the chapter,
  % which is equivalent with min max \sum \beta^t { y' Q y + U' R U ....},
  % adjusting thm as indicated.  Remember that doublex9 solves a min
  % problem.
  
  %[Fs,Ks,Ps,Pst]=doublex9(As,Bs,Cs,Q,Rs,-1/thm);  % flip sign of sig because we have a max problem
  [Fs,Ks,Ps,Pst]=doublex9(As,Bs,Cs,Q,Rs,-1/thm);
  Ks=b^(-.5)*Ks;
  
  Fnew(2,:)=Ks;
  Fnew(1,:)=Fs;
     
  F=Fnew;
  %F=-Fnew;
  P=Ps;  
  P=-Ps;
% Manipulate matrices



P21=P(nz+1:n,1:nz);
P22=P(nz+1:n,nz+1:n);


 % partition C to prepare for computing  new values of sigq, sigv
 
 Cz=C(1:4);    % these are the volatility terms on the natural state z
 
 Cx = -P22\P21*Cz;  % these are the implied volatility terms on the multipliers x
 

 
 
 sigq_2=Cx(1);
 
 
 sigdiff=sigq-sigq_2;
 
end  % this is the end of the sigma loop




%% Now compute objects that characterize the law of motion under the
%% monopolist's robust rule and the approximating model as well as the
%% monopolist's choice of U, W as a function of z, \mu_x:


F1 =F(1,:) ;
F2 =F(2,:);
A1 =A(1:nz,:);
A2 =A(nz+1:n,:);
B1 =Bb(1:nz,:);
B2 =Bb(nz+1:n,:);
C1 =C(1:nz,:);
C2 =C(nz+1:n,:);

%%%======================================

%disp('Law of motion cal M for z, mu_x = '),
 
M=[eye(nz) zeros(nz,n-nz); P21 P22]*[A1-B1*F1 ;A2-B2*F1]*[eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)];
% MW=[eye(nz) zeros(nz,n-nz); P21 P22]*[A1-B1*F1-C1*F2 ;A2-B2*F1-C2*F2]*[eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)];  % this is from
%    % the old compfringe.m program -- not how it builds in the worst case
%    % shock

CM0=C;
%CM1= [eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)]*CM0;  %this is the
%original, which seems wrong -- see \Ep{CMdef} in the monograph
CM1= [eye(nz) zeros(nz,n-nz); P21 P22]*CM0;

%  M, CM  is the law of motion of z_t, \mu_{xt}
CM=CM1;

CM2=CM(nz+1:n,1);


% Monopolist's rule for U, W is
disp('monopolist rule for U, W')
Mrule=[-Fs ; Ks]*[eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)]


%%% following stuff imported from compfringefn.m


%%%%%% new part of program 
 


% Part I.  Use sys to form state space system.  



%
%  (this transformation follows from the transformation 
%  from [z_t, x_t] to [z_t, \mu_{xt}]  .  See chapter 10.  
%
%  get state space system for p, q, Q, mu_u, mu_w, v,   W, w
%                             1  2  3   4     5    6    7 8  (numbers
%                             are position in Y vector)
%
G=[a0  1 -a1 -a1 0 ;    % picks off p
   0  0 0 1 0 ;         % selects q
   0  0 1 0 0 ;          % selects Q
   0  0 0 0 1 ;     % selects \mu_q
   0 1  0 0 0 ;     % selects v;
   Mrule(2,:)];   % picks off W_{t+1}, worst case shock of monopolist
             
 % The observation vector is p, q, Q, mu_q,  v, W                

% now form the state space system M, Ct, G, DD=zeros(9,1)

DD=zeros(6,1);
sys1=ss(M,CM,G,DD,1) ;   % system for approximating model


% now construct the steady state objects


%disp('thetam, thetaf= ' ),thm,thf;
%disp('steady state of price, q, Q')
%  construct steady state

xs=null(eye(5)-M);
xs=xs./xs(1);
%    
% disp(' xs is steady state value of  [1 v  Q q \mu_q ]'),
% xs
% 
% disp('steady state price = ')
% 
% ps=[a0 1 -a1 -a1 0  0]*xs
% disp('steady state q is')
% 
% qs=[0  0 0 1 0 0]*xs
% 
% disp('steady state Q is')
% Qs=[0   0 1 0  0 0]*xs
% 
% disp('steady state xs'),xs
% 
% disp('steady state values of  p, q, Q, mu_u, mu_w, v,   W, w'),
 yss=G*xs;

ps=yss(1);
qs=yss(2);
Qs=yss(3);
muqs=yss(4);
Ws=yss(6);

