% task24gv4.m

function [AA,CC,GG,DD] = task24gv4(A,C,EE,theta)

% revised from task24g.m with changes in r_t^n = e_(r_t^n)'X_t

%  Input: auto-regressive process of X_t, exogenous shocks
%  Output: equation (24) in woodrob3.ps

%  X_t+1 = A*X_t + C*W_t+1 (input)

%  A is a square matrix of dimension na, which is the number of 
%  elements in X_t;

%  EE is a square matrix with the dimension the same as A. 
%  EE'X extracts variables from X_t
%  In the special case of na = 4, 
%  EE=[1 0 0 0;0 1 0 0;0 0 1 0;0 0 0 1] and
%  EE'*X = X

%  EE doesn't have to be identity matrix, but I specify that 
%  the first column must be [1;zeros(na-1,1)]. This column
%  is usually used to get some constants, such as Xstar, rstar.

%  theta is the multiplier for malicous nature

%  Includes Evan's trick for solving the
%  system; Evan's trick works even when does not require that
%  L [I; P] be nonsingular or well conditioned
%
%  theta must be input
%   
%    theta=input('value of theta > 0 = ')

% We study the system L*[phi1 phi2 X] = N*[pai x mu] in task23
% construct L and N in symplectic pencil and calculate P

%  check the inputs

[na,ca] = size(A);
[ne,ce] = size(EE);
nc = size(C,1);

if na ~= ca
   disp('Error in input: A is not square!')
end

if ne ~= ce
   disp('Error in input: E is not square!')
end

if nc ~= na
   disp('Error: Check the dimension of C!')
end


% set parameter values

beta  = .99;       % discount rate
sig = 0.157;       % IS curve parameter
kappa = 0.024;     % Phillips curve parameter
lamx  = 0.047;     % weight on x in objective
lamr  = 0.233;     % weight on r in objective

% set parameters for the transition motion of X

rho = .9999;


% parametize selection vectors
% Given the order of first order conditions, we can only
% make educated selection of vectors from EE ( not able 
% to generalize further)

% e  = [.5 1]';
 ern = EE(:,2); % pulling out r_{t}^{n}, natural rate
 mx = 0;
 ex = EE(:,1)*mx;
 mr = 0;       % mean of rstar
 er = EE(:,1)*mr; 
 e1 = EE(:,3);  % pulling out epsilon_1
 e2 = EE(:,4);  % pulling out epsilon_2

% now set up L and N

L = [beta*sig^(-2)/lamr, 0, zeros(1,na), sig^(-1), 1, zeros(1,na);
   zeros(na,2), eye(na), zeros(na,2), -theta\C*C';
   zeros(1,2+na),beta, zeros(1,1+na);
   0, -beta, zeros(1,2+2*na);
   -beta, kappa*beta, zeros(1,2+2*na);
   -beta*(sig^(-1)*ern+e2),-beta*e1, zeros(na,na+2), beta*A'];

Lorig=L;
L=Lorig*(beta^(-.5));       %  Multiply L by beta^(-.5)
			    %  in order to deduce transformed system


N = [zeros(1,2), sig\(er'-ern')-e2', 0, 1, zeros(1,na);
   zeros(na,2), A, zeros(na,na+2);
   zeros(1,2),-e1', 1, -kappa, zeros(1,na);
   -sig^(-1), -beta, zeros(1,na), 1, zeros(1,1+na);
   -1, 0, -lamx*ex', 0, lamx, zeros(1,na);
   zeros(na,1), zeros(na,3+na), eye(na)];
   
   % call schurg
   
   [V,Lbar,Nbar,alpha,peta,info] = schurg(L,N,0,0,1e-15);
   
   n = size(V,1)/2;
   P = V(n+1:2*n,1:n)/V(1:n,1:n); % P = V21/V11
   
   disp('eigenvalues and reciprocals of eigenvalues of beta-transformed sys')
   
   lambda = alpha./peta % eigenvalues
   1./lambda
   
   %  Compute some equilibrium objects:
   
   LVinv=Lbar*inv(V) 	 % coefficients on LHS
   
   NVinv=Nbar*inv(V)     % coefficients on RHS
   
   
   check1=V*inv(Lbar)*Nbar*inv(V);
   check2=inv(L)*N;
   
   check1-check2
   
   
   
   NVinv*[eye(2+na); P]     % coefficients on Z 
   
   (L\N)*[eye(2+na) ; P]    %  solution for Z_{t+1} chi_{t+1}
  					%  as function of Z_t, chi_t
   
   %  form the state space system
   %
   nn=2+na;    % fix this later   to automate the dimension setting
   
   %  Evan's trick for solving for Ao
   temp1=[eye(nn);P];
   Aoevan=(L*temp1)\(N*temp1);
 
   Aoo=(L\N)*[eye(nn);P]
   Ao=Aoo(1:nn,1:nn);C2=[0 0 0; 0 0 0; C];
   			% changed from Chao because now there are three shocks
   % Ao
   % Aoevan
   check0=Aoevan-Ao
   
      G=P; D=[0 0 0;0 0 0; zeros(size(C))];  %  changed from Chao
   psi=Aoo(nn+1:2*nn,:);   %  we'll also make this GG later
   
   %  the law of motion for the system is
   %
   %  Z_{t+1} = A_o  Z_t 
   %  chi_{t+1} = psi Z_t
   %
   %  now transform the Ao and psi to redo the discounting
   
   Ao=Ao*beta^(-.5);
   psi=psi*beta^(-.5)    ;
 
   %  Now get formula for chi_{t+1} as function of Z_t
   %  chi_{t+1} = GG Z_t
   %
   GG=psi;         
      
   %  formulas for worst case shock w_{t+1} and the interest rate r_t:
   %  r_t = -(beta*sig^{-1}/ lambda_r) Ao(1,:)* Z_t , i.e., components corresponding to 
                       %  phi_1
   %  w_{t+1} = theta^{-1} C' psi(3:nn,:) *Z_t, i.e., psi components corresponding to mu
   
   
   Gr = -(beta*sig^(-1)/lamr)*Ao(1,:)+er'*[zeros(na,2),eye(na)];
   Gw=theta^(-1)*C'*psi(3:nn,:);  %  component corresponding to mu; 
   Dr=[0 0 0];   Dw=zeros(3,3);      %  changed from Chao for 3 shocks   
   Grn = [0 0 ern'];
   
   % return
   %   notice adjustment for worst case shock, and thus the distorted expectation
   %   operator
   
   %  check two ways to get formula for distorted law of motion of state
   %
   %    X_{t+1} = A X_t + theta^{-1} C C' Ao(mu comps)
   %    X_{t+1} = Ao Z_t
   %  check to see whether last two rows are equal 
   %  This now works!   
   
   check3=[zeros(na,2) A] + theta\C*C'*psi(3:nn,:);  
   check4=Ao(3:nn,:);
   check5=check3-check4
   
   
   %  the state space system for Z_t, \chi_t is then
   %  Z_{t+1} = Ao Z_t + C2 v_{t+1}
   %  chi_t = P Z_t + D v_{t+1}   
   %  r_t = Gr Z_t + Dr v_{t+1}
   %  w_{t+1} = Gw z_t + Dw v_{t+1}
   %  rn_t = Grn z_t
   % Form the entire state space system
   
   AA=Ao; CC=C2; GG=[P;Gr;Gw;Grn];  DD=[D;Dr;Dw;Dr];
   
   AA=[Ao(1:2,:);zeros(length(A),2) A];  % set AA to pick off
       % true law of motion for state, not distorted
       % law
   
   %  Note: the observables are (in order) pi (1), x (1), mu (4), r (1)
   %  w_{+1} (3), and r^n_t (1) making a total of 11 observables
   %  the y's from the impulse responses will thus have 11 rows and
   %  we'll have to use the tensors to pull off the appropriate impulse response
   
