%  testrobust2
%  Modified to experiment with the scaling of the objective function in the
%   robust filter (factor of 2 issues)
%
%  Program to check out Lars's conjecture about the Bayesian interpretation
%  of the robust filter
%
%  The test model is
%
%  x_{t+1} = a x_t + c e_{1,t+1}
%
%  y_{t+1} = x_t + d e_{2,t+1}
%
% so A=a, C=c, G=1, D=1 in Lars's notation

a=.9;c=2;d=1;g=1;

% set up the state space system
%
A=a; 
C=[c 0];   
G=g;
D=[0 d];
H=.2;
bet=1;
thet=10;





%gg=1;hh=1;
[Kr,Sig,DelSig]=rfilter(A,C,G,D,H,bet,thet)
[Kr,Kphi,Sig,DelSig]=rfilter2(A,C,G,D,H,bet,thet); 
[Fo,Ko,Po,Pto]=olrprobust(1,A',G',H',C*C',D*D',-(1/thet))  ; %  This checks out



Ncan0=(1/thet)*(C-Kr*D)'*((A-Kr*G)'+H'*Kphi)*H'*H
Ncan=(1/thet)*(C-Kr*D)'*((A-Kr*G)'+H'*Kphi'*(A-Kr*g))*H'*H
Ncan2=(1/thet)*(C-Kr*D)'*(A-Kr*G+H'*Kphi')*(inv(Sig)+H'*H)

%% first, to compute the worst case shock, solve a control problem:

%  Max_w_t \sum .5 eH'He - .5 \thet w'w
%  s.t.
%        e_{t+1} = (A-Kr*G)e + (C-K*D)w
%
% worst case model is w=N e, value function is e'pe
%
%  We should have f=N


[f,p] = olrp(1,A-Kr*G,(C-Kr*D),.5*H'*H,-.5*thet*eye(2))

pp=H'*((1/thet)*H*Sig*H'+1)*H
po=2*inv(pp)






 %olrp(beta,A,B,Q,R,W)


% now create Lars's stacked system
m=2;  % number of shocks
Sigi=2*p;  

% first compute the worst case shock w_t = N e_{t-1}:
N=inv((thet*eye(m)-(C-Kr*D)'*Sigi*(C-Kr*D)))*(C-Kr*D)'*Sigi*(A-Kr*G);  
-f



%N=inv((thet*eye(m)-(C-Kr*D)'*Sigi*(C-Kr*D)))*(C-Kr*D)'*Sigi*(A-Kr*G);  
%-f


A1=C*N;
A2=(C-Kr*D)*N;
G2=D*N;

% Now form the bloated innovation covariance matrix:

JJ=inv(eye(m)-(thet^(-1))*(C-Kr*D)'*Sigi*(C-Kr*D));

%  Factor it:

J=chol(JJ);
J=J';  
%   J=eye(2);  % Note J=eye(2) gives the same answer


C1=C*J;
C2=(C-Kr*D)*J;

%% Now form the augmented system for the Bayesian interpretation 
%% This will be the system AA, CC, GG, DD
A11=A;
A12=A1;
A21=0; 
A22=A-Kr*G+A2;

AA=[A11 A12;A21 A22];

CC=[C1;C2];

GG=[G G2];

DD=D*J;

%% Now compute the ORDINARY Kalman filter

V1=CC*CC'; V2=DD*DD'; V3=CC*DD';


[k,s] = kfilter(AA,GG,V1,V2,V3)

[k1,s1,dels1]=rfilter(AA,CC,GG,DD,[1 0],1,100000000)

% compute the covariance matrix:

omegabig=GG*s*GG'+V2;

% to remind outselves of the robust K-filter: [Kr,Sig,DelSig]=rfilter(A,C,G,D,H,bet,thet)

omegalittle=G*DelSig*G'+D*D';   %note that we use the "distorted" benchmark model for x_t (or x_0)

%[Kr,Sig,DelSig]=rfilter(A,C,G,D,H,bet,thet);

% report some stuff


Kr
k(1)-k(2)
A22

s
Sig

omegabig
omegalittle  
disp('these match, as required')
return
%% Second try: form my system

At=[A+C*N  -C*N; Kr*G+Kr*D*N A-Kr*G-Kr*D*N];
Ct=[C;Kr*D];
Gt=[G+D*N -D*N];
Dt=D;

[kt,sigt]=kfilter(At,Gt,Ct*Ct',Dt*Dt',Ct*Dt')

%% third try: form my version of the x_t, e_t system


% JJ=inv(eye(m)-(thet^(-1))*(C-Kr*D)'*Sigi*(C-Kr*D));
% 
% %  Factor it:
% 
% J=chol(JJ);
% J=J';   


% C1=C*J;
% C2=(C-Kr*D)*J;

As=[A C*N;
    0 A-Kr*G+(C-Kr*D)*N];
Cs=[C;(C-Kr*D)]*J;
Gs=[G D*N];
Ds=D*J;

[ks,sigs]=kfilter(As,Gs,Cs*Cs',Ds*Ds',Cs*Ds')


N
Ncan
N./Ncan


