function[v,pf,con]=value1(a,b,c,r,q,f,beta,nnc)
% function[v,pf,con]=value1(a,b,c,r,q,f,beta,nnc)
% computes pf and con in the value function 
%  v(x) = x'*pf*x + con
% and also the unconditional expectation 
%
% v = E v(x) 
%
%  where v(x) = x'*(r+f'*q*f)*x + beta E v(x')
%
%     x' = (a-b*f)*x + c w
%
%  where w is i.i.d. with mean zero and E w w'=I.
%  nnc is the location of the constant in the state vector.
%  If there is no constant, set nnc=0.
%  See also value2.m.  

af=a-b*f;         				% closed loop system matrix
rf=r+f'*q*f;					   % return function under f

pf=doublej(beta^(.5)*af',rf);  % compute x'*pf*x piece of value function
con=(beta/(1-beta))*trace(pf*c*c');  %  compute constant

sigx=doublej(af',c*c');        % compute unconditional covariance
if nnc==0
   mux=zeros(max(size(a)));
   else
mux=sstate(af,nnc);           % compute unconditional mean
end
v=trace(pf*(sigx+mux*mux'))+con;  % compute unconditional value