function [fv,worstv,v]=valueneng(beta,msig,A,B,C,R,Q)

% outputs:
% dv is the D(v), robustness-augmented covariance matrix;
% tv is the T(D(v)), fixed point
% fv is the F(D(v)), the robust decision rule;
% v is the value function, given the iteration and
% convergence, we expect that v=tv;
% worstv is the worst case shock by the evil nature;
% dv2, tv2,fv2 and v2 are corresponding ones for 2nd algorithm

% inputs:
% n is the dimension of the state vector;
% beta is the rate of time preference 
% theta is the robustness parameter, positive

% msig is the absolute value of sig passed in;
% msig=inv(theta) is the robust parameter;
% A,B,C is the corresponding matrix in x'=A x + B u + C w

% Q and R matrices are the return matrix in the form of 
% r(x,u)=-x'Rx-u'Qu for the instantaneous return function;
% note that it is orthogonalized already;

n=size(A,1);
dd=1;

    tol=1e-10;
    it=1; 
    maxit=95000;    
    small=1e-5;
    v=small*eye(n);
    
[rc,cc]=size(C);

while ((dd > tol) & (it<=maxit))
  
  % D operator
  dv=v+msig*v*C*inv(eye(cc)-msig*C'*v*C)*C'*v; 
  
  tv=R+beta*A'*(dv-beta*dv*B*inv(Q+beta*B'*dv*B)*B'*dv)*A;
  dd=max(max(abs(tv-v)));
  
  it=it+1;
  v=tv;
  
  if it>=maxit
    disp('WARNING: Iteration limit reached in loop');
    disp('');
  end
end

fv=beta*inv(Q+beta*B'*dv*B)*B'*dv*A;

worstv=msig*inv(eye(cc) -msig*C'*v*C) *C'*v*(A-B*fv);
    
%fv=fv';
