% woodford3.m  
% modified version of woodford2 for purpose of doing single theta
%  sargent's modification of Stijn's woodford.m
%  comments are included where modifications have been made
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program computes the Woodford model
% Robustness in forward looking models (no filtering)
% Stijn Van Nieuwerburgh
% First version: January 27, 2001 (stijn)
% Current version: Februar 16, 2001 (stijn)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Set parameter values
     
%th=input('theta =  ')
     sig =0.157 ;    % coefficient on output gap eq.
     kap =0.024 ;    % slope Phillips curve
     lamg=0.047 ;    % preference parameter
     gs= 0.1    ;    % gamma star
     rs= 0.1    ;    % r star 
     lamr=0.233 ;    % preference parameter
     b   =0.95  ;    % discount factor
     thnr=1000000000; % no robustness theta
     %theta  = [20,22,25,27,30,33,35,40,45,50,60,70,80,90,100,150,200,250,500,750,1000,5000,10000,100000,thnr] ;    % robustness parameter
     rho1=0   ;    % persistence in the eps1 process
     rho2=0   ;    % persistence in the eps2 process
     s1=0       ;    % standrd dev of the eps1 process 
     s2=0      ;    % standrd dev of the eps2 process
     rhorn=0.9  ;    % persistence in the natural rate process rn
%  for j=1:length(theta);
    
    
% The state vector for this economy is 6-dimensional
% [1 rn(t) eps1(t) eps2(t) gamma(t) pi(t)]
% The model specified for the natural interest rate implies:
% rn(t)=rhorn*rn(t-1)+sig*(1-rhorn)*epsg
% the model for the shocks to the implementabilbity conditions is:
% eps1(t)=rho1*eps(t-1)+s1*GWN(t)
% Define the following matrices
     
     A= [1 0 0 0 0 0 ; 0 rhorn 0 0 0 0; 0 0 rho1 0 0 0 ; 0 0 0 rho2 0 0 ;...
             0 -1/sig 1/(sig*b) -1 1+(kap/(sig*b)) -1/(sig*b);...  % tjs put - sign on third element
             0 0 -1/b 0 -kap/b 1/b];
     Bb= [0; 0; 0; 0; (1/sig); 0];
     %   C= [0 ;sig*(1-rhorn) ;s1; s2; 0; 0];  % there should be
     %                                           three shocks, not one
     C =[0 0 0;
         sig*(1-rhorn) 0 0;
         0 s1 0;
         0 0 s2; 0 0 0; 0 0 0];   % tjs put this in
     
     B= [Bb C];
     R= [lamr zeros(1,3);zeros(3,1) -b*eye(3)*th];  %tjs added eye(3) because there are 3 shocks
     Q=- [lamg*(gs^2)+lamr*(rs^2) 0 0 0 -lamg*gs 0;0 0 0 0 0 0;0 0 0 0 0 0;0 0 0 0 0 0; -lamg*gs 0 0 0 lamg 0;0 0 0 0 0 1];

% Define the number of 'jump variables', here gamma and pi.
     nx=2;
     n= max(size(Q));
     nz=n-nx;

% Solve the optimal linear regulator from the above matrices
     [F P]= olrp(b,A,B,Q,R);

% Manipulate matrices

P21=P(nz+1:n,1:nz);
P22=P(nz+1:n,nz+1:n);
F1 =F(1,:) ;
F2 =F(2:4,:);     % tjs changed this to reflect the 3 shocks
A1 =A(1:nz,:);
A2 =A(nz+1:n,:);
B1 =Bb(1:nz,:);
B2 =Bb(nz+1:n,:);
C1 =C(1:nz,:);
C2 =C(nz+1:n,:);

% Form the matrices M and MM

M=[eye(nz) zeros(nz,n-nz); P21 P22]*[A1-B1*F1 ;A2-B2*F1]*[eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)];
MW=[eye(nz) zeros(nz,n-nz); P21 P22]*[A1-B1*F1-C1*F2 ;A2-B2*F1-C2*F2]*[eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)];

MM11=M(1:nz,1:nz);
MM12=M(1:nz,nz+1:n);
MM21=M(nz+1:n,1:nz);
MM22=M(nz+1:n,nz+1:n);


% Compute the ARMA representations for controls an perturbations
% Compute the ARMA representations for controls an perturbations
FF= -F*[eye(nz) zeros(nz,n-nz);-inv(P22)*P21 , inv(P22)];
FF11=FF(1,1:nz);
FF12=FF(1,nz+1:n);
FF21=FF(2,1:nz);
FF22=FF(2,nz+1:n);

AR11=FF12*(MM22\FF12');
MA11=FF11;
MA12=FF12*(MM21-(MM22\FF12')*FF11);
disp('The ARMA representation for u is:');
disp('u(t)=');disp(AR11);disp('* u(t-1) + ');disp(MA11);disp('* z(t) + ');disp(MA12); disp('* z(t-1) .');

AR21=FF22*(MM22\FF22');
MA21=FF21;
MA22=FF22*(MM21-(MM22\FF22')*FF21);
disp('The ARMA representation for w is:');
disp('w(t+1)=');disp(AR21);disp('* w(t) + ');disp(MA21);disp('* z(t) + ');disp(MA22); disp('* z(t-1) .');

% Following material added by stijn, 02/16/01

% first form the volatility matrix for z_t, mu_{xt}  by premultiplying

Ct=[eye(nz) zeros(nz,n-nz); P21 P22]*C;

%  get state space system for 1,rn, eps1, eps2, mu_x (2-dimensional)

G=[-inv(P22)*P21 , inv(P22) ;   % selects inflation and output gap gamma
   FF11 , FF12 ;   % selects policy variable u=interest rate r
   FF21 , FF22 ;   % selects policy variable perturbation w      
   0 0 0 0 1 0 ;   % selects first \mu_x 
   0 0 0 0 0 1];   % selects second \mu_x 


% now form the state space system M, Ct, G, zeros

DD=zeros(n,3);   % tjs changed 1 to 3 (number of shocks)
sys1=ss(M,Ct,G,DD,1) ;   % system for approximating model
sys2=ss(MW,Ct,G,DD,1);   % worst case model
 
%  now compute impulse response functions for state vars with respect
% to the shock in eta.  


%  the following has been changed by tjs because now there are three shocks
%  get impulse response w.r.t. first shock 
%  note: impulse puts impulse response of j'th variable to nth shock  in y(:,j,n) 

nn=32;
y1=impulse(sys1,nn);  y1=y1(2:nn,:,1);
y2=impulse(sys2,nn);  y2=y2(2:nn,:,1);



figure(1)

tt=[0:nn-2];
subplot(6,1,1)
stairs(tt,y1(:,1)),ylabel('\pi')
subplot(6,1,2)
stairs(tt,y1(:,2)),ylabel('\gamma')
subplot(6,1,3)
stairs(tt,y1(:,3)),ylabel('r')
subplot(6,1,4)
stairs(tt,y1(:,4)),ylabel('w')
subplot(6,1,5)
stairs(tt,y1(:,5)),ylabel('\mu_1')
subplot(6,1,6)
stairs(tt,y1(:,6)),ylabel('\mu_2')

% Let us now run some simulations

TT=1000;
randn('seed',0);
u=randn(TT,3);    %  tjs changed this for 3 shocks
 
 
%  construct steady state
 
 xss=null(eye(n)-M);
 xs=xss./xss(1);
 
xinitial=xs; 
xinitial(nz+1:n)=0;   % start from zero shadow price for implementability multipliers
yy1=lsim(sys1,u,[],xinitial);

figure(2)
subplot(3,1,1)
plot(yy1(:,1),'-g'),ylabel('\pi')
subplot(3,1,2),
plot(yy1(:,2),'-b'),ylabel('\gamma')
subplot(3,1,3)
plot(yy1(:,3),'--r'),ylabel('r')
 
figure(3)
subplot(3,1,1)
plot(yy1(:,4),'-g'),ylabel('w')
subplot(3,1,2)
plot(yy1(:,5),'-b'),ylabel('\mu_1')
subplot(3,1,3)
plot(yy1(:,6),'--r'),ylabel('\mu_2')
 
return

varr(j)=var(yy1(:,3));
%  end;


return

figure(4);
scatter(1:length(theta),varr);
set(gca,'xticklabel',[20,30,50,100,750,1e+9]);
